;==============================================================================================
;**** Hermit's 1 Rasterline Tracker **** 
;   2013  Hermit Software Hungary    (Mihaly Horvath)   hermit@upcmail.hu
;---------------------------------------------------------------------------------------------
;compiler to use: 64tass v1.5x (minimum rev361 due to the new relocation method with 'union')
;usage of illegal opcodes requires the '-i' option for 64tass (read it's manual for more info)
;TODO is at the end of this source

;constants
Year=2013
ORMversion="0.9"
VRAM=$0400
CRAM=$d800
repspd1=$0d ;first keypress repeat delay in frames (x20ms)
repspd2=$02 ;continuous keyrepeat delay (repetition speed) in frames (x20ms)
CursorFlashSpd1=1 ;flashing-speed of cursor in jamming mode
CursorFlashSpd2=3 ;flashing-speed of cursor in editing mode
PALframes1second=50 ;frames in 1 second on PAL machine
PALfullrast = 312   ;$138	;amount of rasterrows (PAL) VIC produces
NTSCframes1second=60 ;how many frames a second is - used for clock timer (PAL machines: 50, NTSC machines: 60)
NTSCfullrast = 263   ;$107	;amount of rasterrows (PAL) VIC produces
defaultColour= 12 ;tracker's default colour
defaultMenuColour= 5 ;menu's default colour

;zeropage used by irq:
zpvar1=2
zpvar2=3
zpvar3=4
zptr1=5
zptr2=7
zptr3=9
pt1ptL=11
pt1ptH=13
pt2ptL=15
pt2ptH=17
pt3ptr=19

;zeropage used by main loop only
zpvar4=21
zpvar5=22
zptr4=23

;==============EXECUTER=================
*= $0801
		.word ss,1
		.null $9e,^start ;SYS command
ss		.word 0

*=2061
SafeStart sei
		  jmp hotStrt

start	sei                   ;we don't want lost cycles by IRQ calls :)
		
		lda $ba			;the default device might be where SID-Maker was loaded from
		cmp #13+1		;if above 13, force 8 as default
		bcs +
		cmp #8
		bcs ++			;if below 16 and greater than / equal to 8, no forcing, leave as it is
+		lda #8			;default device number if exporter was loaded from non-disk drive
+		sta filedialog.devicen+1
		
		jsr initzak
		jsr inittbl
		
		jsr PALorNTSC ;detect if PAL or NTSC machine, store 0/1 result at a memory location 'PALNTSC'
		;set pitch-detector for machine-type
		lda #<FREQBCK
		ldx #>FREQBCK
		ldy PALNTSC
		sty player.machtype
		sty SIDmachtype
		bne +
		lda #<FREQBCK_NTSC
		ldx #>FREQBCK_NTSC
+		sta p_fback1+1
		stx p_fback1+2
		sta p_fback2+1
		stx p_fback2+2
		
		jsr detectSIDtype
		stx SIDtype
		
hotStrt	lda #$35
		sta 1
		lda #0
		sta playmode
		sta menumode
		sta helpmode
		
startEd	jsr resetimer
		jsr player.INIT	;player.INIT TUNE

;-this code mustn't be split by a $100 boundary because it needs to be cycle-exact
;sync cmp $d012             ;scan for begin rasterline (A=$11 after first return)
;     bne *-3       ;wait if not reached rasterline #$11 yet
;     ldy #8        ;the walue for cia timer fetch & for y-delay loop
;     sty $dc04     ;CIA Timer will count from 8,8 down to 7,6,5,4,3,2,1
;     dey           ;Y=Y-1 (8 iterations: 7,6,5,4,3,2,1,0)
;     bne *-1       ;loop needed to complete the poll-delay with 39 cycles
;     sty $dc05     ;no need Hi-byte for timer at all (or it will mess up)
;     sta $dc0e,y   ;forced restart of the timer to value 8 (set in dc04)
;     lda #$11      ;value for d012 scan and for timerstart in dc0e
;     cmp $d012     ;check if line ended (new line) or not (same line)
;     sty $d015     ;switch off sprites, they eat cycles when fetched
;     bne sync      ;if line changed after 63 cycles, resyncronize it!     
.cwarn *>=$0900,"The timing-critical code has been split by page boundary! You should rearrange it to get cycle-exact timing."

;------------------------
softEdi	sei
		lda #$35
		sta $01
		lda #3
		sta $dd00
		lda #$1b
		sta $d011
		lda #$16
		sta $d018
		lda #$08
		sta $d016
		lda #0
		sta $d020
		sta $d021
		jsr ackIRQ
		lda #<irq
		ldx #>irq
		sta $fffe
		stx $ffff
		lda #0
		sta namemode
		jsr drawScreen
		cli

;------------------------
;-this code mustn't be split by a $100 boundary because it needs to be cycle-exact
mainloop			
		lda menumode
		beq +
		jmp Menu
		+
syncsig	lda #0
		beq + ;synchronization to screen-refreshment (vblank)
		jsr dispTable
		jsr dispOrderList
		lda #0
		sta syncsig+1
+		jmp mainloop

chkPlay	
		lda playmode
		beq +
		dec $d020
		jsr player.PLAY
		inc $d020
		jsr timecount
+		rts

ackIRQ	lda #$7f
		sta $dc0d
		sta $dd0d
		lda $dc0d
		lda $dd0d
		lda #1
		sta $d01a
		sta $d019
		lda #$fe
		sta $d012
		lda #<dummirq
		sta $fffa		;NMI (restore key) pointer setting
		lda #>dummirq
		sta $fffb
		rts

;---------------------------------------------------------
;.align $100 ;cycle-timing sensitive code comes, avoiding boundaries that add 1 cycle
irq		pha				;store sensitive registers and bank status
		txa
		pha
		tya
		pha
		
     ;lda $dc04   ;check timer A, here it jitters between 7...1
     ;eor #7      ;A=7-A so jitter will be 0...6 in A
     ;sta corr+1  ;self-writing code, the bpl jump-address = A
corr nop ;bpl *+2     ;the jump to timer (A) dependent byte
     bit 0 ;cmp #$c9    ;if A=0, cmp#$c9; if A=1, cmp #$c9 again 2 cycles later
     ;cmp #$c9    ;if A=2, cmp#$c9, if A=3, CMP #$EA 2 cycles later
     ;bit $ea24   ;if A=4,bit$ea24; if A=5, bit $ea, if A=6, only NOP
     ;bit $ea24   ;if A=4,bit$ea24; if A=5, bit $ea, if A=6, only NOP
     
     ldx #15
     dex
     bne *-1
     bit 0
     
		jsr chkPlay

		lda #1
		sta syncsig+1

		;inc $d020
		jsr dispCursor
		jsr dispVal
		jsr timecount.dispTimer
		jsr dispPatt

		lda $dc01
		cmp #$ff ;if no key pressed
		bne cangoEd
		lda #1
		sta cangoEd+1 ;enable keys
cangoEd	lda #0 ;self-written - avoids returning back to editor instantly for F8
		beq +
		jsr KeyHandler
		+
		
		jsr followplay
		;dec $d020
		
retirq1	lda #1			;more compatible than asl $d019
		sta $d019
		pla
		tay
		pla
		tax
		pla
		rti

dummirq 			;NMI (Restore key) ends here, used for page-up in VICE
		bit $dc0d
		inc restsig+1	;signs for keyhandler that restore was pressed. routine zeroes it
		rti
		
DispCursor
		lda #1
		sta $d015
		lda #0
		sta $d01b
		lda #<(ValueCursor/$40)
		sta VRAM+$3F8

		ldx window
		bne chPtCur
Setti1C	lda #4
		clc
		adc diginum
		sta curx+1
		ldy setYpos
		lda setYdat,y
		sta cury+1
		jmp showcur

chPtCur	cpx #2
		bpl chTbCur
		ldx ptrack
		lda trkCptr,x
		sta VRAM+$3F8
		lda trCposX,x
		cpx #2
		bne +
		clc
		adc diginum
+		sta curx+1
		lda trkYpos
		clc
		adc #3
		sta cury+1
		jmp showcur
trkCptr .byte <(NoteCursor/$40),<(NoteCursor/$40),<(ValueCursor/$40) 
trCposX	.byte 11,19,27
tblCptr .byte <(ValueCursor/$40),<(ValueCursor/$40),<(NoteCursor/$40) ,<(NoteCursor/$40),<(NoteCursor/$40),<(NoteCursor/$40)  
tbCposX .byte 34,35,37, 37,37,37

chTbCur	bne chS2Cur
		ldx diginum
		jsr getTbIndex
		beq TbFxCur
		lda player.notemode
		beq TbFxCur
		lda tblCptr,x
		sta VRAM+$3F8
		lda tbCposX,x
		sta curx+1
		jmp +
TbFxCur	txa			;display table-FX row cursor
		clc
		adc #34
		sta curx+1
+		lda tblYpos
		clc
		adc #3
		sta cury+1
		jmp showcur

chS2Cur	cpx #4
		bpl chOlCur
		lda diginum
		clc
		adc #38
		sta curx+1
		lda se2Ypos
		clc
		adc #17
		sta cury+1
		jmp showcur

chOlCur	lda OliXpos
		sta zpvar1
		asl
		clc
		adc zpvar1
		adc #1
		adc diginum
		sta curx+1
		lda otrack
		clc
		adc #21
		sta cury+1

showcur
cury	lda #0			;self-written variable ;cursor y coordinate (0..25..28)
		asl
		asl
		asl
		clc
		adc #49
		sta $d001
curx	lda #0			;self-written variable	 ;cursor x coordinate (0..39)
		asl
		asl
		asl
		php
		clc
		adc #$18
		sta $d000
		lda #$c0
		bcc no9bit1
		ora #$01
no9bit1 plp
		bcc no9bit2
		ora #$01
no9bit2 sta $d010
		;display cursor colour
		lda #<($100-CursorFlashSpd1)
jammode ldy #1			;self-written variable ;if 0 - edit, if 1 - jam
		bne +			;write/jam mode of keyboard
		lda #<($100-CursorFlashSpd2) ;if edit-mode, flash faster
+		clc
		adc curcolc+1	;added later (rev.108) to make the difference between jam/edit mode more percievable
		sta curcolc+1	;cursor color flasher
		bpl norescc
		lda #<(curcolend-curColour)-1
		sta curcolc+1
norescc
curcolc ldy #0			;self-written variable ;cursor colour counter
		lda curColour,y
		sta $d027
		rts

getTbIndex
		lda tblYpos
		clc
		adc tablepos
		tay
		lda player.TblCol1,y
		rts

window	.byte 1 ;current panel/window: 0-setting1, 1-pattern-editor, 2-table, 3-setting2, 4-orderlist
ptrack	.byte 0 ;current track in pattern-editor (0/1/2)
otrack	.byte 0 ;current track in orderlist-editor
oliXpos .byte 0 ;X (data) position in orderlist
diginum	.byte 0 ;1st or 2nd digit
trkYpos	.byte 0 ;Y position in tracks
setYpos	.byte 0 ;Y position in 1st settings
tblYpos .byte 0 ;Y position in table
se2Ypos	.byte 0 ;Y position in 2nd settings

setYdat .byte 2,3,5,6,7,9,10,11,13,14,15,17,18,19
setYdat_end
setYbck .byte 1,1,2,3,4,5,5,6,7,8,8,9,10,11,11,12,13 ;inverse of setYdat (for Pattern-cursor 'left'...)

curColour
		.byte $00,$0b,$06,$04,$0e,$0c,$03,$0f,$0d,$07,$07,$01;cursor colours to flash
		.byte $07,$01,$01,$07
		.byte $01,$07,$07,$0d,$0f,$0a,$0c,$08,$02,$0b,$09,$00
		.byte $09,$00,$00,$09
curcolend

;---------------------------------------------------------
KeyHandler
		jsr shcbget		;shift & CBM key requester
		lda #7
		sta zpvar2
keylp1	ldy zpvar2
		lda keyrows,y
		sta $dc00
		lda $dc01
		cmp #$ff
		beq skiprow
		ldx #7
keylp2	ldy zpvar2
		lda $dc01
		and keycols,x
		bne nextkey
		stx zpvar1
		tya
		asl
		asl
		asl
		clc
		adc zpvar1
		asl ;x2 (.word)
		tay
		
		cpy #30			;Left-Shift?   -not to count shift/cbm keys now, done already in 'shcbget'
		beq nextkey		;check next
		cpy #104		;Right-Shift?
		beq nextkey		;check next
		cpy #122		;C= key?
		beq nextkey		;check next
zprvkey	cpy #0			;self-written - compare previous key to current, if different, reset repeat-counter
		beq +			;if the same key is pressed, no counter-reset
		ldx #repspd1	;no pressed key, repeatcounter resets
		stx repecnt		;using X to preserve A
+		sty zprvkey+1
		
		ldx namemode	;name-typing mode?
		beq +
		jmp nametyper
		+
		
		lda menumode
		beq +
		jmp MenuKeyHandler
		+
		
		lda keyjmp+0,y
		sta keyJump+1
		lda keyjmp+1,y
		sta keyJump+2
keyJump	jmp dummykey	;self-written - jump
nextkey	dex
		bpl keylp2
skiprow	dec zpvar2
		bpl keylp1
restsig lda #0			;self-written variable ;signs that there was restore pressed
		beq nokey		;check if restore key was pressed, and perform Page-UP if was
		lda #0
		sta restsig+1
		lda menumode
		and helpmode
		beq +
		jmp helpgup
+		inc shiftsi
		jmp slasher
nokey	lda #repspd1	;no pressed key, repeatcounter resets
		sta repecnt
		ldx #0
		lda playmode
		bne +
		lda instPos
		sta tblcnt
		stx CHN3+4 
prvnote	lda #0			;self-written - previous note
		beq +
		;ldy ptrack
		;lda mul7,y
		;tay
		lda #0
		sta CHN1+0 ;sta SID+0,y
		sta CHN1+1 ;sta SID+1,y
		sta CHN2+0
		sta CHN2+1
+		sta auditioning
		sta unbounce
		rts

shcbget	ldx #0			;get status of SHIFT and CBM keys
		stx shiftsi
		stx cbmsig
		ldy #2
-		lda modkeyc,y
		sta $dc00
		lda $dc01
		and modkeyr,y
		bne +
		inc shiftsi,x
+		dey
		bne +
		inx
+		bpl -
		rts
modkeyc .byte $7f,$bf,$fd;columns of CBM, Right-SHIFT, Left-Shift
modkeyr .byte $20,$10,$80;rows of CBM, Right-SHIFT, Left-Shift
shiftsi	.byte 0
cbmsig  .byte 0

repeatex				;repeat-counter routine, return only if repeat can be done (shouldn't affect X or Y registers)
		lda repecnt		;keyrepeat-counter
		bne norsrep		;check, if repeat counter is 0
		lda #repspd2	;default repeat-speed 2
		sta repecnt		;if 0, reset counter to repeatspeed 2
noret	pla 			;getting back Program-counter from stack? (injected by Soci)
		pla				;causing '2-level rts'?
		rts				;repeat not allowed here
norsrep cmp #repspd1	;check, of repeat counter is initial value (repeatspeed 1)
		bne repcan		;if no, then just decrease the repeat counter
		dec repecnt		;if yes, decrease by one, and reset Z-flag - repeat possible
		rts				;repeat allowed here
repcan	dec repecnt
		bne noret		;if reaches 0, a repeat occurs, next round will reset to speed 2
		rts				;repeat allowed here
repecnt	.byte 0

keyjmp ;keyboard-jumptable                                                   ;- 01 - 02 - 04 - 08 - 10 - 20 - 40 - 80 ---
 .word deleter ,return  ,cursori ,f7er    ,f1er    ,f3er    ,f5er    ,cursrdn ; del |retu|crri| F7 | F1 | F3 | F5 |crdn|fe
 .word key3sub ,keyWsub ,keyAsub ,key4sub ,keyZsub ,keySsub ,keyEsub ,nextkey ;  3  | W  | A  | 4  | Z  | S  | E  |lSH |fd
 .word key5sub ,keyRsub ,keyDsub ,key6sub ,keyCsub ,keyFsub ,keyTsub ,keyXsub ;  5  | R  | D  | 6  | C  | F  | T  | X  |fb
 .word key7sub ,keyYsub ,keyGsub ,key8sub ,keyBsub ,keyHsub ,keyUsub ,keyVsub ;  7  | Y  | G  | 8  | B  | H  | U  | V  |f7
 .word key9sub ,keyIsub ,keyJsub ,key0sub ,keyMsub ,keyKsub ,keyOsub ,keyNsub ;  9  | I  | J  | 0  | M  | K  | O  | N  |ef
 .word pluser  ,keyPsub ,keyLsub ,minuser ,dotter  ,coloner ,dummykey,commasub;  +  | P  | L  | -  | .  | :  | @  | ,  |df
 .word inskeye ,dummykey,semicol ,homer   ,nextkey ,equaler ,slasher ,slasher ;pound| *  | ;  |HOME|rSH | =  | up | /  |bf
 .word key1sub ,bckarrow,ctrler  ,key2sub ,spacer  ,nextkey ,keyQsub ,stopper ;  1  |back|CTRL| 2  |SPC | C= | Q  |stop|7f

keycols .byte $01,$02,$04,$08,$10,$20,$40,$80
keyrows .byte $fe,$fd,$fb,$f7,$ef,$df,$bf,$7f


.align $40
ValueCursor
.byte %11111111,%10000000,%00000000
.byte %10000000,%10000000,%00000000
.byte %10000000,%10000000,%00000000
.byte %10000000,%10000000,%00000000
.byte %10000000,%10000000,%00000000
.byte %10000000,%10000000,%00000000
.byte %10000000,%10000000,%00000000
.byte %10000000,%10000000,%00000000
.byte %11111111,%10000000,%00000000
.byte %00000000,%00000000,%00000000
.byte %00000000,%00000000,%00000000
.byte %00000000,%00000000,%00000000
.byte %00000000,%00000000,%00000000
.byte %00000000,%00000000,%00000000
.byte %00000000,%00000000,%00000000
.byte %00000000,%00000000,%00000000
.byte %00000000,%00000000,%00000000
.byte %00000000,%00000000,%00000000
.byte %00000000,%00000000,%00000000
.byte %00000000,%00000000,%00000000
.byte %00000000,%00000000,%00000000

.align $40
NoteCursor
.byte %11111111,%11111111,%11111111
.byte %10000000,%00000000,%00000001
.byte %10000000,%00000000,%00000001
.byte %10000000,%00000000,%00000001
.byte %10000000,%00000000,%00000001
.byte %10000000,%00000000,%00000001
.byte %10000000,%00000000,%00000001
.byte %10000000,%00000000,%00000001
.byte %11111111,%11111111,%11111111
.byte %00000000,%00000000,%00000000
.byte %00000000,%00000000,%00000000
.byte %00000000,%00000000,%00000000
.byte %00000000,%00000000,%00000000
.byte %00000000,%00000000,%00000000
.byte %00000000,%00000000,%00000000
.byte %00000000,%00000000,%00000000
.byte %00000000,%00000000,%00000000
.byte %00000000,%00000000,%00000000
.byte %00000000,%00000000,%00000000
.byte %00000000,%00000000,%00000000
.byte %00000000,%00000000,%00000000


;===============================================================================================================
		.cerror *>EXEstarter,"TOO MUCH CODE BEFORE EXE-STARTER DOESN'T FIT IN MEMORY. MOVE SOME CODE TO UPPER FREE AREAS!"

;FROM HERE TILL END OF PLAYER DATA DO NOT INSERT ANY CODE!!! ALL SPACE IS KEPT FOR SID TUNE ITSELF
;================================= BASIC-starter for executable export ==========================================
EXEstarterSize = $200
		*= SIDHEADER-EXEstarterSize
EXEstarter	.block			;executable-SID startercode
decozptr=$e8
compzptr=$ea
titlerow=3	;row of tune-title
subturow=8	;row of subtune-info
restarow=11	;row ow restart-info
timesrow=17	;row of playtime/rastertime info
editorow=24	;row of editor-info 
		.logical $0801	;set logical compilation address
BASICbe .word ss2,Year
		.null $9e,^start2 ;Sys 2064
ss2		.word 0
		.byte 0,0,0		;to align to $0810
;address 2064 ($0810) - INITIALIZATION AT PROGRAM START - set IRQ handlers, screen, and main VIC registers
start2	sei
		lda #$35
		sta 1
		lda #$35
		sta 1
-		lda $d012	;detect if machine is PAL/NTSC
-		cmp $d012
		beq -		;waiting 1 rasterline's time (no matter CPU speed)
		bmi --		
		ldx #0		;0 for NTSC
		cmp #(PALfullrast//$100)-1 ;$37	;PAL machine?
		bne +
		inx			;1 for PAL
+		stx MachTyp ;set to 0 if NTSC, 1 if PAL
		
		lda #0 ;EXEBGROUND ;init screen
		sta $d020
		sta $d021
		lda #$17		;small character set
		sta $d018
		ldy #0
		sty $d015		;switch off sprites to prevent rastertime-cycles being stolen from CPU
-		lda #$20		;clear screen
		sta $0400,y
		sta $0500,y
		sta $0600,y
		sta $0700,y
		lda #3 ;EXETEXTCOL
		sta $d800,y
		sta $d900,y
		sta $da00,y
		sta $db00,y
		iny
		bne -
		;copy SID-tune to correct location
EXEsrcL lda #<(BASICbe+EXEstarterSize+SIDHEADERSIZE)+(player.tunedata_end-PLAYERADDR)		;set source-address  -self-written code
		sta compzptr
EXEsrcH lda #>(BASICbe+EXEstarterSize+SIDHEADERSIZE)+(player.tunedata_end-PLAYERADDR)		;self-written code
		sta compzptr+1
EXEtgtL lda #<player.tunedata_end			;set target-address  -self-written code
		sta decozptr
EXEtgtH lda #>player.tunedata_end			;self-written code
		sta decozptr+1
		ldy #0
EXEcopy lda (compzptr),y;read data from source
		sta (decozptr),y;write data to destination
		lda compzptr+0
		bne +
		dec compzptr+1
+		dec compzptr+0
		lda decozptr+0
		bne +
		dec decozptr+1
+		dec decozptr+0
		lda compzptr+0
		cmp #<EXEstarter_logical_end
		lda compzptr+1
		sbc #>EXEstarter_logical_end
		bcs EXEcopy
;play the tune
		sei
		ldy #39			;display author-info
-		lda #$40 ;"*"		;decorate title
		sta $0400+(titlerow-1)*40,y
		sta $0400+(titlerow+1)*40,y
		lda player.authorinfo,y;SIDtitletx,y
		sta $0400+titlerow*40,y
		lda resttxt,y	;tune-restart info
		jsr lowerca		;handle lowercase
		sta $0400+restarow*40,y
		lda timetxt,y	;playtime/rastertime info
		jsr lowerca		;handle lowercase
		sta $0400+timesrow*40,y
		lda madewith,y
		jsr lowerca		;handle lowercase
		sta $0400+editorow*40,y
		dey
		bpl -
		dey
		bpl -
		
initExe	lda #0
		sta minute
		sta second
		jsr PLAYERADDR
		lda #PALframes1second
		sta framect
-		lda $dc01
		cmp #$ff
		bne -
		
plyloop	lda #$fe
		cmp $d012
		bne *-3
		ldx #9 ;#4
		dex
		bne *-1
		bit 0
		bit 0
		
		inc $d020
		jsr PLAYERADDR+3
		dec $d020
		
counTim	dec framect
		bpl dispTim
		ldx MachTyp		;PAL/NTSC timing?
		lda framesec,x
		sta framect
		inc second
		lda second
		cmp #60
		bne dispTim
		lda #0
		sta second
		inc minute
dispTim	lda minute		;display playback-timetxt
		jsr decimdi
		sty $0400+timesrow*40+10
		sta $0400+timesrow*40+11
		lda second
		jsr decimdi
		sty $0400+timesrow*40+13
		sta $0400+timesrow*40+14
chSpace	lda #$7F
		sta $dc00
		lda $dc01
		cmp #$ef
		beq initExe ;wait9ra
		jmp plyloop

decimdi	ldy #$2f		;input:A, output: decimal digits in Y and A
		sec
-		iny
		sbc #10
		bcs -
		adc #$3a
		rts

lowerca	cmp #$60		;input: Accu, output: Accu
		bcc +			;handle lowercase
		and #$1f
+		rts

subttxt	.text "Select subtune with +/- . Current: 00/00"
resttxt	.text "You can restart the tune with SPACE key."
timetxt	.text "PlayTime: 00:00 , Rastertime:01 (max.01)"
minute	.byte 00 ;counts minutes
second	.byte 00 ;counts seconds
framect	.byte PALframes1second ;counts frames for timer - initial value is set based on machine-type
MachTyp .byte 1 ;PAL=1, NTSC=0
framesec .byte NTSCframes1second, PALframes1second
madewith .text "Composed with 1 rasterline tracker v",ORMversion,"    " ;editor-info row, can be overwritten by different player-types

EXEstarter_logical_end
.here			;restore physical address
.bend
.cerror *>(EXEstarter+EXEstarterSize),"YOU MUST ENSURE MORE PLACE ('EXESTARTERSIZE') FOR RUNNABLE-SID STARTER CODE!"

;==========================================================================
.cerror *>SIDHEADER,"Too much code before player-code!"
SIDHEADERSIZE = $7c
		*= PLAYERADDR-SIDHEADERSIZE;...SID file-format header
SIDHEADER
;SID v1 header - big endian (!!!) WORD (byte-pair) values most of the time
;--------------
		.text "PSID"	;+00 magicID: 'PSID' or 'RSID'	(RSID stands for strict SID that requires full, real C64 environment or full emulation
		.byte $00,$02	;+04 version ;can either be 0001 or 0002. Headers of version 2 provide additional fields. RSID and PSID v2NG files must have 0002 here.
		.byte $00,$7C	;+06 dataOffset ;offset from the start of the file to the C64 binary data area. fixed size of the header, this is either 0x0076 for version 1 and 0x007C for version 2.
		.byte $00,$00	;+08 loadAddress ;C64 memory location where to put the C64 data. 0 means the data are in original C64 binary file format, must NOT be less than $07E8 in RSID files.
SIDinitadd .byte >player.INIT,<player.INIT;+0A initAddress ;start address of the machine code subroutine that initializes a song,accepting the contents of the 8-bit 6510 Accumulator as the song number parameter. 0 means the address is equal to the effective load address.
SIDplayadd .byte >player.PLAY,<player.PLAY;+0C playAddress ;start address of the machine code subroutine that can be called frequently to produce a continuous sound. 0 means the initialization subroutine is expected to install an interrupt handler, which then calls the music player at some place. This must always be true for RSID files.
SIDstamount .byte $00,$01;+$0E songs ;The number of songs (or sound effects) that can be initialized by calling the init address. The minimum is 1. The maximum is 256.
SIDdefasubt .byte $00,$01;+$10 startSong ;The song number to be played by default. This value is optional. It often specifies the first song you would hear upon starting the program is has been taken from. It has a default of 1.
SIDtimerTyp .byte $00,$00,$00,$00;+12 LONGWORD speed ;32 bit big endian number. Each bit in 'speed' specifies the speed for the corresponding tune number, i.e. bit 0 specifies the speed for tune 1. 0 bit specifies vertical blank interrupt (50Hz PAL, 60Hz NTSC), and a 1 bit specifies CIA 1 timer interrupt (default 60Hz).
SIDtitletx .fill $20,0	;+$16 <title> ;32 byte long zero terminated ISO 8859-1 ASCII character strings. zero byte will always be put into the last byte of each string. So the maximum number of available free characters is 31.
SIDauthrtx .fill $20,0	;+$36 <author>
SIDreleatx .fill $20,0	;+$56 <released> (also known as <copyright>)
;SID v2 header extensions:
;-------------------------
SIDflags .byte %00000000, %00100100;+$76 flags ;16 bit big endian number containing the following bitfields: bit0-binary data format (0-built-in player,1-Compute MUS player must be merged), bit1-(0-C64 compatible,1-PlySID specific/C64 BASIC flag for RSID), bit2..3-video standard(01-PAL,10-NTSC), bit4..5-SID model (00-?,01-6581,10-8580,11-both), other bits:0.
		.byte $00		;+$78 startPage (relocStartPage) ;8 bit number. If 'startPage' is 0, the SID file is clean, i.e. it does not write outside its data range within the driver ranges. If 'startPage' is 0xFF, there is not even a single free page, and driver relocation is impossible.if 'startPage' is 0x1E, this free memory range starts at $1E00.
		.byte $00		;+$79 pageLength (relocPages) ;8 bit number indicating the number of free pages after 'startPage'. If 'startPage' is not 0 or 0xFF, 'pageLength' is set to the number of free pages starting at 'startPage'. If 'startPage' is 0 or 0xFF, 'pageLength' must be set to 0.
		.byte $00,$00	;+$7A reserved ;This is a 16 bit number and is reserved and should be set to 0.
;--------------------------------------------------------------------------------------------------------
;+7C	<data> ;Version 2 of the SID header ends here. This offset is the start of the binary C64 data.
;----------------------------------------------
;*= PLAYERADDR-2 ;used by SID-export 
;LOADADDR .byte <PLAYERADDR,>PLAYERADDR ;relocator will NOT modify this - used for safety-saving

*=PLAYERADDR
.include "player.asm" ;music program + data



;==============================subroutines=================================
;.align $100 ;makes sure the code won't be overwritten by editor


f1er	jsr repeatex
;		lda offs_pos
;		beq +
;		lda #0           ;display orderlist-position in case of pick-up too
;		sta seqcnt
;		lda #defptlen-1
;		sta ptncnt
;		jsr dispOrderList
+		lda cbmsig
		sta followp+1
		jsr PtPlayOFF
		jsr player.INIT
		lda #1
		sta playmode
		jsr resetimer
		lda shiftsi
		beq retF1
plyMark lda #defptlen-1
		sta ptncnt ;don't use pick-up for F2 playback 
		lda marker
		sta seqcnt
		jsr player.setPtPt
retF1	rts

PtPlayOFF		
		lda patplyOFF+0
		sta player.patplay+0
		lda patplyOFF+1
		sta player.patplay+1
		rts

patplyON
.logical player.patplay
 bne player.retplay		;pattern-playback code overwritten in player
.here
patplyOFF inc seqcnt ;normal playback code

f3er	lda shiftsi
		bne stopper
		jsr repeatex
		lda cbmsig
		sta followp+1
		lda patplyON+0
		sta player.patplay+0
		lda patplyON+1
		sta player.patplay+1
		lda #1
		sta playmode
		jsr resetimer
		lda #1
		sta spdcnt
		lda #defptlen-1
		sta ptncnt
		lda selptn+0
		sta player.ptn1pL+1	;3
		sta player.ptn1pH+1	;3
		lda selptn+1
		sta player.ptn2pL+1	;3
		sta player.ptn2pH+1	;3
		lda selptn+2
		sta ptn3pt		;3
		rts
stopper	jsr repeatex
		lda cbmsig
		sta followp+1
pauscnt	lda playmode 
		eor #1
		sta playmode
		bne +
		lda tblcnt
		sta tbcntemp
		lda #0
		sta SID+0
		sta SID+1
		sta SID+7
		sta SID+8
		sta SID+14
		sta SID+15
		rts
+		lda tbcntemp
		sta tblcnt
f3reter	rts
playmode .byte 0 ;play / pause
tbcntemp .byte 0 ;temporary storage for table-counter

f5er	lda cbmsig
		beq +
		lda #0
		sta window
		rts
+		ldx #1
		lda shiftsi
		beq +
		ldx #4
+		stx window
		lda #0
		sta diginum
		rts
		
f7er	jsr repeatex
		lda cbmsig
		beq +
		ldx #3
		stx window
		lda #0
		sta diginum
		rts
+		lda shiftsi
		beq +
		inc menumode
		rts
+		ldx #2
		stx window
		rts

bckarrow
toggflw	lda shiftsi
		beq ffwd
		jsr repeatex
		lda playmode
		beq +
		lda followp+1	;don't set follow-play mode while stopped, it would disable pattern-editing
		eor #1
		sta followp+1
+		rts
ffwd	jsr player.PLAY
		jsr timecount
		jsr player.PLAY
		jsr timecount
		jsr player.PLAY
		jsr timecount
		lda #0
		sta $d020
+		rts

;----------------------------------------
cursrdn	jsr repeatex
curkeyY	lda window		;X register shouldn't be used from here (needed for 'slasher' Page-UP/DN) 
		bne chPatDn
		lda shiftsi
		bne set1up
set1dwn	ldy setYpos
		iny
		cpy #(setYdat_end-setYdat)
		bmi +
		ldy #0
		jmp +
set1up	ldy setYpos
		dey
		bpl +
		ldy#(setYdat_end-setYdat)-1
+		sty setYpos
		rts
chPatDn	cmp #2
		bpl chTbDn
		lda shiftsi
		bne trkup
trkdown	lda trkYpos
		cmp #16
		bpl scrPtDn
		inc trkYpos
		rts
scrPtDn	lda ptnpos
		cmp #defptlen-17
		bpl retudn2
		inc ptnpos
		rts
trkup	lda trkYpos
		beq scrPtUp
		dec trkYpos
		rts
scrPtUp	lda ptnpos
		beq retudn2
		dec ptnpos
retudn2	rts
chTbDn	bne chS2Dn
		lda shiftsi
		bne tblup
tbldown	lda tblYpos
		cmp #12
		bpl scrTbDn
		inc tblYpos
		rts
scrTbDn	ldy tablepos
		iny
		cpy #(tablemax+1)-12
		bne +
		ldy #0
		sty tblYpos
+		sty tablepos
		rts
tblup	lda tblYpos
		beq scrTbUp
		dec tblYpos
		rts
scrTbUp	ldy tablepos
		bne +
		lda #12
		sta tblYpos
		ldy #(tablemax+1)-12
+		dey
		sty tablepos
		rts
chS2Dn	cmp #4
		bpl oliDn
		lda shiftsi
		bne set2up
		ldy se2Ypos
		iny
		cpy #3
		bne +
		ldy #0
+		sty se2Ypos
		rts
set2up	ldy se2Ypos
		dey
		bpl +
		ldy #2
+		sty se2Ypos
		rts
oliDn	lda shiftsi
		bne oliUp
		ldy otrack
		iny
		cpy #3
		bne +
		ldy #0
+		sty otrack
		rts
oliUp	ldy otrack
		dey
		bpl +
		ldy #2
+		sty otrack
retudn	rts

slasher	jsr repeatex
		ldx #8
		lda window
		bne pgLoop
		dex
pgLoop	lda window
		cmp #4 ;orderlist?
		bne +
		jsr olirigh
		jmp ++
+		jsr curkeyY
+		dex
		bne pgLoop
		rts

;----------------------------------------
cursori	jsr repeatex
		lda window
		bne chPtRig
setcuri	lda shiftsi
		bne crleft
		lda diginum
		bne goptnwi
		inc diginum
		rts
goptnwi	lda #1
		sta window
		lda #0
		sta ptrack
		sta diginum
		ldy setYpos
		lda setYdat,y
		sec
		sbc #3
		bpl +
		lda #0
+		sta trkYpos
		rts
crleft	lda window
		bne returi1
		lda diginum
		beq returi1
		dec diginum
returi1	rts
chPtRig	cmp #2
		bpl chTbRig
ptncuri	lda shiftsi
		bne trcback
		lda ptrack
		cmp #2
		beq +
		inc ptrack
		rts
+		lda diginum
		bne gotable
		inc diginum
		rts
gotable	lda #2
		sta window
		lda #0
		sta diginum
		lda trkYpos
		cmp #13
		bmi ++
		sec
		sbc #14
		bpl +
		lda #0
+		sta se2Ypos
		lda #3
		sta window
		rts
+		sta tblYpos
		rts

trcback	lda ptrack
		beq gosetwi
		cmp #2
		bne +
		lda diginum
		beq +
		dec diginum
		rts
+		dec ptrack
		rts
gosetwi	lda #0
		sta window
		lda #1
		sta diginum
		ldy trkYpos
		lda setYbck,y
		sta setYpos
		rts
chTbRig	bne chS2Rig
		lda shiftsi
		bne +
		lda diginum
		cmp #5
		bpl returi2
		inc diginum
		rts
+		ldx diginum
		beq goptnw2
		jsr getTbIndex
		beq +
		lda player.notemode
		beq +
		cpx #2
		bmi +
		lda #1
		sta diginum
		rts
+		dec diginum
returi2	rts
chS2Rig	cmp #4
		bpl olirigh
		lda shiftsi
		bne se2left
		lda diginum
		bne +
		inc diginum
+		rts
se2left	lda diginum
		beq +
		dec diginum
		rts
+		lda #1
		sta window
		sta diginum
		lda se2Ypos
		clc
		adc #14
		sta TrkYpos
		lda #2
		sta ptrack
		rts
goptnw2	lda #1
		sta window
		lda #2
		sta ptrack
		lda #1
		sta diginum
		lda tblYpos
		sta trkYpos
		rts
olirigh	lda shiftsi ;X register shouldn't be used here, because counts PageUP/DN amount
		bne olileft
		lda diginum
		bne +
		inc diginum
		rts
+		lda oliXpos
		cmp #12
		bpl +
		dec diginum
		inc oliXpos
		rts
+		lda seqpos
		cmp #(seqlen-1)-12
		bcs returig
		inc seqpos
		rts
olileft	lda diginum
		beq +
		dec diginum
		rts
+		lda oliXpos
		beq +
		inc diginum
		dec oliXpos
		rts
+		lda seqpos
		beq returig
		dec seqpos
returig	rts

;--------------------------------------
ctrler	jsr repeatex
		lda window
		cmp #1
		bne ctrlret
		lda shiftsi
		bne trkback
trkforw	ldy ptrack
		iny
		cpy #3
		bne +
		ldy #0
+		sty ptrack
		rts
trkback	ldy ptrack
		dey
		bpl + 
		ldy #2
+		sty ptrack
ctrlret	rts

homer	jsr repeatex
		ldx #0
		lda window
		bne +
		stx setYpos
		rts
+		cmp #2
		bpl ++
		lda trkYpos
		bne +
		stx ptnpos
+		stx trkYpos 
		rts
+		bne ++
		lda tblYpos
		bne +
		stx tablepos
+		stx tblYpos
		rts
+		cmp #4
		bmi homeret
		ldx #0
		lda oliXpos
		bne +
		stx seqpos
+		stx oliXpos
		stx diginum
homeret	rts

spacer	jsr repeatex
		lda shiftsi
		bne +
		lda jammode+1
		eor #1
		sta jammode+1
		rts
+		lda window
		cmp #4
		bne +
		lda seqpos
		clc
		adc oliXpos
		sta marker
		rts
+		cmp #1
		bne +
		lda #0 ;lda #<State0	;init FSM
		sta spdcnt ;sta FSMstat+1
		jsr getPtIndex
		sta ptncnt
		lda #1 ;tempo+1
		sta spdcnt
		lda #1
		sta playmode
+		cmp #2
		bne spcretu
		lda playmode
		bne spcretu
		jsr getTbIndex
		sty instPos
		sty tblcnt
spcretu	rts

return	lda window
		cmp #1
		bne chTbRet
		jsr repeatex
		lda ptrack
		cmp #2
		bne retret1 ;returet
		jsr getPtIndex
		tay
		lda (pt3ptr),y
		bne goInstr
findEmp	ldy #tablemax		;find empty row 
-		ldx player.TblCol1,y
		bne +
		ldx player.TblCol2,y
		bne +
		ldx player.TblCol3,y
		cpx #endcmd
		bne +
		tya
		dey
		bne -
		beq goInstr
+		;cmp #tablemax
		;bcs goInstr
		;clc
		;adc #1 ;after $fe
goInstr	cmp #tablemax+1
		bcs retret1
		ldx #2
		sta instPos
		cmp #tablemax-10
		bcc +
		sec
		sbc #tablemax-12
		tax
		lda #tablemax-10
+		cmp #2
		bcs +
		tax
		lda #2
+		sec
		sbc #2
		sta tablepos
		stx tblYpos
		lda #0
		sta diginum
		inc unbounce
		lda #2
		sta window
retret1	rts
chTbRet	cmp #2
		bne chOlRet
tblAudi	lda playmode
		bne returet
		lda unbounce
		bne returet
		lda auditioning
		bne auditio		;audition runs already?
		inc auditioning
		lda shiftsi
		bne audiCur
		lda cbmsig
		bne audiTop
audiIns	ldy instPos
		jmp +
audiTop	ldy tablepos
		jmp +
audiCur	jsr getTbIndex
+		sty tblcnt
		rts
auditio	jmp player.playTbl ;auditioning
auditioning .byte 0 ;if aiditioning is in process
unbounce .byte 0 ;wait until return is released
instPos	.byte 0 ;position (table-index) of last selected instrument
		
chOlRet	cmp #4
		bne returet
		jsr repeatex
		lda #1
		sta window
		lda otrack
		sta ptrack
		lda #0
		sta ptnpos
		sta trkYpos
		sta diginum
		lda seqpos
		clc
		adc oliXpos
		tay
;		lda shiftsi
;		bne setpall
;		lda otrack
;		bne +
;		jmp setpt1
;+		cmp #1
;		bne +
;		jmp setpt2
;+		jmp setpt3
setpall	jsr setpt1
		jsr setpt2
setpt3	lda player.seq3,y
		sta selptn+2
returet	rts
setpt1	lda player.seq1,y
		sta selptn+0
		rts
setpt2	lda player.seq2,y
		sta selptn+1
		rts

;---------------------------------------------
deleter	jsr repeatex
		lda shiftsi
		beq +
		jmp insert
+		lda window
		cmp #1
		bne chTbDel
		jmp delePtn
chTbDel	cmp #2
		bne chOlDel
		jsr getTbIndex
		ldx cbmsig ;delete a complete row
		beq +
		jsr delcolu
		jmp tbldown
+		sty zptr1
		cpy #tablemax
		beq delcolu
		inc zptr1
-		lda player.TblCol1+1,y
		sta player.TblCol1,y
		bne +			;fx?
		ldx player.TblCol3+1,y
		cpx #tblcnt		;jump-fx?
		bne +			;correct jump-fx address
		ldx player.TblCol2+1,y
		cpx zptr1
		bcc +
		dex
		txa
		jmp ++
+		lda player.TblCol2+1,y
+		sta player.TblCol2,y
		lda player.TblCol3+1,y
		sta player.TblCol3,y
		iny
		cpy #tablemax+1
		bne -
		dey
		jsr delcolu
corrPt3	ldy #0
-		lda player.ptn3,y
		beq +
		cmp zptr1
		bcc +
		sec
		sbc #1
		sta player.ptn3,y
+		iny
		bne -
		rts
delcolu	lda #0
		sta player.TblCol1,y
		sta player.TblCol2,y
		lda #endcmd ;player end-command
		sta player.TblCol3,y
		rts
chOlDel	cmp #4
		bne delert1 ;deleret
		lda seqpos
		clc
		adc oliXpos
		tay
-		cpy #(seqlen-1)
		bcs +
		iny
		jsr seqread
		dey
		jsr seqwrit
		iny
		bne -
+		lda #0
		jsr seqwrit
delert1	rts
insert	lda window
		cmp #1
		bne chTbIns
		jmp instPtn
chTbIns	cmp #2
		bne chOlIns
		jsr getTbIndex
		sty zptr1
		cpy #tablemax
		beq delcolu
+		ldy #tablemax
-		lda player.TblCol1-1,y
		sta player.TblCol1,y
		bne +			;fx?
		ldx player.TblCol3-1,y
		cpx #tblcnt		;jump-fx?
		bne +			;correct jump-fx address
		ldx player.TblCol2-1,y
		beq +			;don't disturb jumping to zero
		cpx zptr1
		bcc +
		inx
		txa
		jmp ++
+		lda player.TblCol2-1,y
+		sta player.TblCol2,y
		lda player.TblCol3-1,y
		sta player.TblCol3,y
		dey
		cpy zptr1
		bne -
		jsr delcolu
corPt3b	ldy #0
-		lda player.ptn3,y
		beq +
		cmp zptr1
		bcc +
		clc
		adc #1
		sta player.ptn3,y
+		iny
		bne -
		rts
chOlIns	cmp #4
		bne deleret
		lda seqpos
		clc
		adc oliXpos
		ldy #(seqlen-1)
		cmp #(seqlen-1)
		beq +
		sta zpvar1
-		dey
		jsr seqread
		iny
		jsr seqwrit
		dey
		cpy zpvar1
		bne -
+		lda #0
		jsr seqwrit
deleret	rts

DelePtn lda followp+1
		bne deleret
		jsr setPtnPtr
		lda ptrack
		bne + ;track1?
		lda pt1ptL+0
		ldx pt1ptL+1
		jsr deleteP
		lda pt1ptH+0
		ldx pt1ptH+1
		jmp delAdva
+		cmp #1
		bne +  ;track2?
		lda pt2ptL+0
		ldx pt2ptL+1
		jsr deleteP
		lda pt2ptH+0
		ldx pt2ptH+1
		jmp delAdva
+		lda pt3ptr+0 ;track3
		ldx pt3ptr+1
delAdva	jsr deleteP
		lda cbmsig
		beq +
		jsr trkdown ;go down to next row if in-place deletion
+		rts
		
deleteP	sta zptr1+0
		stx zptr1+1
		jsr getPtIndex
		ldx cbmsig	 ;delete only cursor-position
		beq +
		tay
		jsr delProw
		rts
+		tay
-		;cpy #defptlen-1
		beq + ;bcs +
		dey ;iny
		lda (zptr1),y
		iny ;dey
		sta (zptr1),y
		dey ;iny
		bne -
+		jmp delProw
		
InstPtn	lda followp+1
		bne PtInRet
		jsr setPtnPtr
		lda ptrack
		bne + ;track1?
		lda pt1ptL+0
		ldx pt1ptL+1
		jsr insertP
		lda pt1ptH+0
		ldx pt1ptH+1
		jmp insertP
+		cmp #1
		bne +  ;track2?
		lda pt2ptL+0
		ldx pt2ptL+1
		jsr insertP
		lda pt2ptH+0
		ldx pt2ptH+1
		jmp insertP
+		lda pt3ptr+0 ;track3
		ldx pt3ptr+1
insertP	sta zptr1+0
		stx zptr1+1
		jsr getPtIndex
		ldy #0 ;ldy #defptlen-1
		cmp #0 ;cmp #defptlen-1
		beq delProw
		sta zpvar1
-		iny ;dey
		lda (zptr1),y
		dey ;iny
		sta (zptr1),y
		iny ;dey
		cpy zpvar1
		bne -
delProw	lda #0
		sta (zptr1),y
PtInRet	rts

inskeye	jsr repeatex
		jmp insert

pluser	jsr repeatex
		lda octave
		cmp #6
		bpl +
		inc octave
+		rts
minuser	jsr repeatex
		lda octave
		beq +
		dec octave
+		rts

semicol	rts

equaler	jsr repeatex
		lda player.notemode
		eor #1
		sta player.notemode
		rts

;----------------------------------------------
key1sub ldx #$01
		.byte $2c		;skip
key2sub ldx #$02
		.byte $2c		;skip
key3sub ldx #$03
		lda shiftsi
		beq octa123
		jsr repeatex  ;mute/solo tracks
		dex
		lda mutesolo,x
		eor #1
MuteSub	sta mutesolo,x
muter	lda #>SID
		ldy mutesolo,x
		bne +
mutetrk	ldy mul7,x
		lda #0
		sta SID+0,y
		sta SID+1,y
		lda #>dummySID
+		pha
		txa
		asl
		tax
		pla
		ldy plyMutL,x
		sty zptr1+0
		ldy plyMutL+1,x
		sty zptr1+1
		ldy #2
		sta (zptr1),y
		ldy plyMutH,x
		sty zptr1+0
		ldy plyMutH+1,x
		sty zptr1+1
		ldy #2
		sta (zptr1),y
		lda player.p_mute6+2
		sta player.p_mute7+2
		lda player.p_mute1+2
		sta player.zpcode_src+(player.p_mute1-zpcode)+2
		sta player.zpcode_src+(player.p_mute2-zpcode)+2
		lda player.p_mute3+2
		sta player.zpcode_src+(player.p_mute3-zpcode)+2
		sta player.zpcode_src+(player.p_mute4-zpcode)+2
		rts
		
octa123	lda cbmsig  ;octave selection
		bne setocta
		jmp datawriter

mul7	.byte 0,7,14
mutesolo .byte 1,1,1
plyMutL .word player.p_mute1,player.p_mute3,player.p_mute5 ;code to overwrite in player to mute
plyMutH	.word player.p_mute2,player.p_mute4,player.p_mute6

key4sub ldx #$04
		.byte $2c		;skip
key5sub ldx #$05
		.byte $2c		;skip
key6sub ldx #$06
		.byte $2c		;skip
key7sub ldx #$07
setocta	lda cbmsig
		beq +
		jsr repeatex
		dex
		stx octave
		rts
+		jmp datawriter
		
key0sub	ldx #$00
		.byte $2c		;skip
key8sub	ldx #$08
		.byte $2c		;skip
key9sub	ldx #$09
		.byte $2c		;skip
keyAsub	ldx #$0a
		.byte $2c		;skip
keyBsub	ldx #$0b
		.byte $2c		;skip
keyDsub	ldx #$0d
		.byte $2c		;skip
keyEsub	ldx #$0e
		.byte $2c		;skip
keyFsub	ldx #$0f
		jmp datawriter

keyCsub	ldx #$0c
		lda shiftsi
		bne CopyRet
		lda cbmsig
		bne +
		jmp datawriter
+		lda followp+1
		bne CopyRet
		jsr repeatex
CopyCur	lda window		;copy data from cursor-position to buffer
		cmp #1 ;pattern-editor?
		bne CopyRet
CopyPtn	jsr setPtnPtr
		jsr getPtIndex ;into Accu
		tay
		lda ptrack
		bne chCoPt2
		sty NoteBufLen
-		lda (pt1ptL),y
		sta noteBufL,y
		lda (pt1ptH),y
		sta noteBufH,y
		dey
		bpl -
		rts
chCoPt2	cmp #1
		bne CopyPt3
		sty NoteBufLen
-		lda (pt2ptL),y
		sta noteBufL,y
		lda (pt2ptH),y
		sta noteBufH,y
		dey
		bpl -
		rts
CopyPt3	sty pt3bufLen
-		lda (pt3ptr),y
		sta pt3buf,y
		dey
		bpl -
CopyRet	rts

keyVsub	ldx #0+6		;'F' note in lower octave
		lda shiftsi
		bne PastRet
		lda cbmsig
		bne +
		jmp notewriter
+		lda followp+1
		bne PastRet
		jsr repeatex
PasteCu	lda window
		cmp #1 ;pattern-editor?
		bne PastRet
PastePt	jsr setPtnPtr
		jsr getPtIndex ;into Accu
		tay
		lda ptrack
		bne chPaPt2
		ldx noteBufLen
-		lda noteBufL,x
		sta (pt1ptL),y
		lda noteBufH,x
		sta (pt1ptH),y
		dey
		bmi +
		dex
+		bpl -
PaPtEnd	rts
chPaPt2	cmp #1
		bne PastPt3
		ldx noteBufLen
-		lda noteBufL,x
		sta (pt2ptL),y
		lda noteBufH,x
		sta (pt2ptH),y
		dey
		bmi +
		dex
		bpl -
+		rts
PastPt3	ldx pt3bufLen
-		lda pt3buf,x
		sta (pt3ptr),y
		dey
		bmi PastRet
		dex
		bpl -
PastRet	rts

keyGsub	ldx #0+7		;'F#' note in lower octave ;lda #0+8		;'G' note in the lower keyboard octave
		.byte $2c		;skip
keyIsub	ldx #12+12+1	;'C' note in 2nd upper octave ;lda #12+12+1	;'C' note in 2nd upper octave
		.byte $2c		;skip
keyJsub	ldx #0+11		;'A#' note in lower octave ;lda #0+12		;'H' note in lower octave
		.byte $2c		;skip
keyLsub	ldx #12+2		;'C#' note in upper octave at lower row ;lda #12+3		;'D' note in upper octave at lower row
		.byte $2c		;skip
keyMsub	ldx #0+12		;'H' note in lower octave
		.byte $2c		;skip
keyNsub	ldx #0+10		;'A' note in lower octave
		.byte $2c		;skip
keyOsub	ldx #12+12+3	;'D' note in 2nd upper octave ;lda #12+2		;'C#' note in upper octave at lower row
		.byte $2c		;skip
keyPsub	ldx #12+12+5	;'E' note in 2nd upper octave ;lda #12+4		;'D#' note in upper octave at lower row
		.byte $2c		;skip
keyQsub	ldx #12+1		;'C' note in upper octave
		.byte $2c		;skip
keyRsub	ldx #12+6		;'F' note in upper octave ;lda #$ff		;no sound for key R in DMC mode
		.byte $2c		;skip
keySsub	ldx #0+2		;'C#' note in lower octave ;lda #0+3		;'D' note in lower octave
		.byte $2c		;skip
keyTsub	ldx #12+8		;'G' note in upper octave ;lda #0+7		;'F#' note in lower octave 
		.byte $2c		;skip
keyUsub	ldx #12+12		;'H' (B) note in upper octave ;lda #0+11		;'A#' note in lower octave
		.byte $2c		;skip
keyWsub ldx #12+3		;'D' note in upper octave ;lda #0+2		;'C#' note in lower octave - for DMC mode
		.byte $2c		;skip
keyXsub	ldx #0+3		;'D' note in lower octave
		.byte $2c		;skip
keyYsub	ldx #12+10		;'A' note in upper octave ;lda #0+9		;'G#' note in lower octave - for DMC mode
		.byte $2c		;skip
keyZsub	ldx #0+1		;'C' note in lower octave
		.byte $2c		;skip
commasub ldx #12+1		;'C' note in upper octave at lower row
		.byte $2c		;skip
dotter	ldx #12+3		;'D' note in upper octave at lower row
		.byte $2c		;skip
coloner	ldx #12+4		;'D#' note in upper octave at lower row
;		.byte $2c		;skip
;norSdot	lda #12+5		;Y=E E in the higher keyboard octave
		jmp notewriter

keyHsub	ldx #0+9		;'G#' note in lower octave ;lda #0+10		;'A' note in lower octave
		lda shiftsi
		bne HkeyRet
		lda cbmsig
		beq notewriter
invokHl	lda #1			;invoke help instantly
		sta menumode
		sta helpmode
		lda #0
		sta cangohl+1
HkeyRet	rts
		
keyKsub
dummykey rts

;----------------------------- note-writer -------------------------
notewriter
		jsr repeatex
notewri	stx prvnote+1
		txa
		clc
		ldy octave
		adc OCTMULS,y
		cmp #$60
		bcs noteret ;don't allow too big
notewr2	tax
		lda window
		cmp #1
		bne chTbNot
		lda followp+1
		bne noteret
		ldy ptrack
		cpy #2
		bpl noteret
		lda playmode
		bne +
		lda mul7,y
		tay
		jsr getFrqL
		sta CHN1+0,y
		jsr getFrqH
		sta CHN1+1,y
+		lda jammode+1
		bne noteret
		jsr getPtIndex
		tay
		lda ptrack
		bne +
		jsr getFrqL
		sta (pt1ptL),y
		jsr getFrqH
		sta (pt1ptH),y
		jmp trkdown ;advance to next row
+		jsr getFrqL
		sta (pt2ptL),y
		jsr getFrqH
		sta (pt2ptH),y
		jmp trkdown ;advance to next row
noteret	rts
chTbNot	cmp #2
		bne notere2
		lda diginum
		cmp #2
		bmi notere2
		lda playmode
		bne +
		jsr getFrqL
		sta CHN3+0
		jsr getFrqH
		sta CHN3+1
+		jsr getTbIndex
		beq +
		lda jammode+1
		bne +
		jsr getFrqL
		sta player.TblCol2,y
		jsr getFrqH
		sta player.TblCol3,y
+		lda player.TblCol1,y
		ora #1
		sta CHN3+4
notere2	rts ;jmp tbldown

getFrqL	lda PALNTSC
		beq +
		lda FreqTablePalLo,x
		rts
+		lda FreqTableNtscLo,x
		rts
getFrqH	lda PALNTSC
		beq +
		lda FreqTablePalHi,x
		rts
+		lda FreqTableNtscHi,x
		rts

octave .byte 4
OCTMULS .byte 0,12,24,36,48,60,72,84,96;TRANSPOSITION VALUE OF OCTAVES TO ADD/SUBSTRACT FROM NOTE

;----------------------------- data-writer -------------------------
datawriter
		jsr repeatex
		stx wrhxad1+1
		stx wrhxad3+1
		lda window
		bne chPtWri
wriSet1	ldy setYpos
		lda player.tunesettings,y
		jsr writehex
		sta player.tunesettings,y
		jmp player.softIni ;in player.asm
chPtWri	cmp #2
		bpl chTbWri
		lda followp+1
		bne retTbWr ;retdatw
		lda ptrack
		cmp #2
		beq +
		jmp NoteDat ;retdatw
+		jsr getPtIndex
		tay
		lda (pt3ptr),y
		jsr writehex
		sta (pt3ptr),y
		rts ;jmp trkdown ;advance to next row
chTbWri	beq +
		jmp chS2Wri
+		jsr getTbIndex
		beq NumMode
		jmp WfAmode
NumMode	ldx diginum
		cpx #4
		bmi chTbCo2
		lda player.TblCol3,y
		cmp #seqcnt ;test if orderlist-position FX, correct $00..3F to $FF..$3E
		bne ++
		lda player.TblCol2,y
		clc
		adc #1
		jsr writehex
		cmp #(seqlen-1) ;restriction - works only with powers of 2 
		bcc +
		lda #(seqlen-1)
+		sec
		sbc #1
		sta player.TblCol2,y
		rts
+		cmp #ptncnt ;test if pattern-position FX, carrect 1F..00 to 00..1F seamlessly
		bne +
		lda #defptlen-1
		sec
		sbc player.TblCol2,y
		jsr writehex
		sta zptr1
		lda #defptlen-1
		sec
		sbc zptr1
		and #defptlen-1 ;restrict size (only works with power of 2 pattern-lengths!)
		sta player.TblCol2,y
		rts
+		cmp #tblcnt  ;correct overjumping
		bne +
		lda player.TblCol2,y
		jsr writehex
		cmp #tablemax
		bcc wrTbCo2
		lda #tablemax
wrTbCo2	sta player.TblCol2,y
		rts
+		lda player.TblCol2,y
		jsr writehex
		sta player.TblCol2,y
retTbWr	rts
chTbCo2	cpx #2
		bmi WrTbCo1
		lda player.TblCol3,y
		jsr writehex
		ldx player.TblCol1,y
		bne ++ ;if pitch-entry
corTbFx	cmp #$80
		bcs +
		cmp #$1f		;only allow valid SID registers
		bcc ++
		lda #$1f
		jmp ++
+		cmp #zpeffects	;only allow valid effect numbers
		bcs +
		lda #endcmd
+		sta player.TblCol3,y
		cmp #tblcnt  ;correct table-jumping command
		bne +
		lda #0
		sta player.TblCol2,y
+		rts
WfAmode	lda player.notemode
		bne +
		jmp NumMode
+		lda diginum
		cmp #2
		bpl NoteDat ;retdatw
WrTbCo1	lda player.TblCol1,y
		jsr writehex
		sta player.TblCol1,y
		bne retTbWr
		lda player.TblCol3,y
		jmp corTbFx ;if 0, take care of Fx-values
		
chS2Wri	cmp #4
		bpl chOlWri
		ldy se2Ypos
		lda player.sldstart,y
		jsr writehex
		sta player.sldstart,y
		jmp player.softIni
chOlWri	lda seqpos
		clc
		adc oliXpos
		tay
		jsr seqread
		jsr convSeq
		jsr writehex
		asl
		asl
		asl
		asl
		asl ;*32
		jsr seqwrit
retdatw	rts

NoteDat	lda NoteOfX,x
		bne +
		jmp notewr2
+		cmp #$ff
		beq retNdat
		tax
		jmp notewri 
retNdat rts

NoteOfX	.byte 12+12+4	;X=0 D# in the second higher keyboard octave
		.byte 0			;X=1 no note to play, using for NOP placement
		.byte 12+2		;X=2 C# in the higher keyboard octave
		.byte 12+4		;X=3 D# in the higher keyboard octave
		.byte $ff		;X=4 no note for key 4
		.byte 12+7		;X=5 F# in the higher keyboard octave
		.byte 12+9		;X=6 G# in the higher keyboard octave
		.byte 12+11		;X=7 A# in the higher keyboard octave
		.byte $ff		;X=8 no note for key 8
		.byte 12+12+2	;X=9 C# in the second higher keyboard octave
		.byte 0			;X=A no note assigned for key A, using it for NOP
		.byte 0+8		;X=B G in the lower keyboard octave
		.byte 0+5		;X=C E in the lower keyboard octave
		.byte 0+4		;X=D D# in the higher keyboard octave
		.byte 12+5		;X=E E in the higher keyboard octave
		.byte $ff		;X=F no sound produced on F key for jamming

seqread ldx otrack
		bne +
		lda player.seq1,y
		rts
+		cpx #1
		bne +
		lda player.seq2,y
		rts
+		lda player.seq3,y
		rts

seqwrit ldx otrack
		bne +
		sta player.seq1,y
		rts
+		cpx #1
		bne +
		sta player.seq2,y
		rts
+		sta player.seq3,y
		rts

writehex ror diginum
		bcs writehex2
;merge typed 1st hexa digit with existing data
		rol diginum
		and #$0f
		sta wrhxad2+1
wrhxad1 lda #0			;self-written
		asl
		asl
		asl
		asl
wrhxad2 ora #0			;self-written
		inc diginum
		rts
writehex2			;merge typed 2nd hexa digit with existing data
		rol diginum
		and #$f0
wrhxad3 ora #0			;self-written
		rts

;-------------------------- display routines -----------------------
drawScreen
		ldx trackerColour
		jsr clearScreen
		
		ldy #3*40
-		lda headertxt-1,y
		;ora #$80
		sta VRAM-1,y
		dey
		bne -
		ldy #4*40
-		lda setting2txt-1,y
		sta VRAM+16*40-1,y
		dey
		bne -

		lda #<VRAM+3*40
		ldx #>VRAM+3*40
		sta zptr1+0
		stx zptr1+1
		ldx #12
		stx zpvar1
		ldx #0
-		ldy #0
-		lda settingtxt,x
		sta (zptr1),y
		inx
		iny
		cpy #4
		bne -
		lda #40
		jsr incZpt1
		dec zpvar1
		bpl --

		ldy #39
-		lda VRAM+2*40,y
		ora #$80
		sta VRAM+2*40,y
		lda #$ef
		sta VRAM+20*40,y
		;lda #3
		;sta CRAM,y
		dey
		bpl -
		lda PALNTSC
		bne +
		ldy #3
-		lda NTSCtext,y
		sta VRAM+1*40+6,y
		dey
		bpl -
+		rts
NTSCtext .text "NTSC"

clearScreen ;X holds colour
		ldy #0
-		lda #" "
		sta VRAM,y
		sta VRAM+$100,y
		sta VRAM+$200,y
		sta VRAM+$300,y
		txa
		sta CRAM,y
		sta CRAM+$100,y
		sta CRAM+$200,y
		sta CRAM+$300,y
		iny
		bne -
		rts
trackerColour .byte defaultColour
menuColour .byte defaultMenuColour

dispVal	;display data of tune-header
		ldx #(player.displayed_end-player.tunesettings)-1
-		lda datPosL,x
		sta zptr1+0
		lda datPosH,x
		sta zptr1+1
		lda player.tunesettings,x
		pha
		lsr
		lsr
		lsr
		lsr
		tay
		lda hexdigi,y
		ldy #0
		sta (zptr1),y
		pla
		and #$0f
		tay
		lda hexdigi,y
		ldy #1
		sta (zptr1),y
		dex
		bpl -
dPtnNum	ldx #5
-		lda ptNuPos,x
		sta zptr1+1
		dex
		lda ptNuPos,x
		sta zptr1+0
		txa
		lsr
		tay
		lda selptn,y
		jsr convSeq
		pha
		;lsr
		;lsr
		;lsr
		;lsr
		;tay
		lda #"0"+$80 ;lda invdigi,y
		ldy #0
		sta (zptr1),y
		pla
		and #$0f
		tay
		lda invdigi,y
		ldy #1
		sta (zptr1),y
		dex
		bpl -
		rts
		
datPosL	.byte <(VRAM+2*40+4),<(VRAM+3*40+4),<(VRAM+5*40+4),<(VRAM+6*40+4),<(VRAM+7*40+4),<(VRAM+9*40+4),<(VRAM+10*40+4),<(VRAM+11*40+4),<(VRAM+13*40+4)
		.byte <(VRAM+14*40+4),<(VRAM+15*40+4),<(VRAM+17*40+4),<(VRAM+18*40+4),<(VRAM+19*40+4),<(VRAM+17*40+38),<(VRAM+18*40+38),<(VRAM+19*40+38)
datPosH	.byte >(VRAM+2*40+4),>(VRAM+3*40+4),>(VRAM+5*40+4),>(VRAM+6*40+4),>(VRAM+7*40+4),>(VRAM+9*40+4),>(VRAM+10*40+4),>(VRAM+11*40+4),>(VRAM+13*40+4)
		.byte >(VRAM+14*40+4),>(VRAM+15*40+4),>(VRAM+17*40+4),>(VRAM+18*40+4),>(VRAM+19*40+4),>(VRAM+17*40+38),>(VRAM+18*40+38),>(VRAM+19*40+38)
		
selptn .byte 0,0,0 ;patterns selected for editing
ptNuPos .word VRAM+2*40+12,VRAM+2*40+20,VRAM+2*40+28

dispPatt
		lda #<VRAM+3*40+8
		sta zptr1+0
		lda #>VRAM+3*40+8
		sta zptr1+1
		jsr setPtnPtr		;set zeropage pointers pointing to the selected patterns
		lda #defptlen-1
		sec
		sbc ptnpos
		sta zpvar1	;counts index
		lda #17
		sta zpvar2	;counts rows
pDiLoop	lda #defptlen-1
		sec
		sbc zpvar1
		pha
		lsr
		lsr
		lsr
		lsr
		tay
		lda hexdigi,y
		ldy #0
		sta (zptr1),y
		ldy #8
		sta (zptr1),y
		ldy #8*2
		sta (zptr1),y
		pla
		and #$0f
		tay
		lda ptNdigi,y
		ldy #1
		sta (zptr1),y
		ldy #1+8
		sta (zptr1),y
		ldy #1+8*2
		sta (zptr1),y
disPt3	ldy zpvar1
		lda (pt3ptr),y
		bne +
		lda #"."
		ldy #19
		sta (zptr1),y
		jmp ++
+		pha
		lsr
		lsr
		lsr
		lsr
		tay
		lda hexdigi,y
		ldy #19
		sta (zptr1),y
		pla
		and #$0f
		tay
		lda hexdigi,y
+		ldy #20
		sta (zptr1),y
dP1Note	ldy zpvar1
sw_pt1l	lda (pt1ptL),y
		tax
sw_pt1h	lda (pt1ptH),y
		jsr GetNote
		lda noteCh1,x
		ldy #3
		sta (zptr1),y
		lda noteCh2,x
		iny
		sta (zptr1),y
		lda noteCh3,x
		iny
		sta (zptr1),y
dP2Note	ldy zpvar1
		lda (pt2ptL),y
		tax
		lda (pt2ptH),y
		jsr GetNote
		lda noteCh1,x
		ldy #11
		sta (zptr1),y
		lda noteCh2,x
		iny
		sta (zptr1),y
		lda noteCh3,x
		iny
		sta (zptr1),y
		dec zpvar1
dPlMark	lda playmode
		beq +
		lda tempo
		cmp #1
		bmi +
		cmp spdcnt
		beq +++
+		lda #" "
		ldy zpvar1
		;iny
		cpy ptncnt
		bne +
		lda #"<"
+		ldy #6
		sta (zptr1),y
		ldy #6+8
		sta (zptr1),y
		ldy #6+8*2-1
		sta (zptr1),y
+		lda #40
		jsr incZpt1
		dec zpvar2
		beq +
		jmp pDiLoop
+		ldy #2
-		lda #"T"+$80
		ldx mutesolo,y
		bne +
		lda #"-"
+		ldx mutesolopos,y
		sta VRAM+2*40+8,x
		;sta VRAM+2*40+9,x
		dey
		bpl -
		rts
mutesolopos .byte 0,8,16

setPtnPtr
		ldy #4
-		lda ptnpIni,y
		ldx zptrNum,y
		sta pt1ptL+1,x
		ldx ptnpNum,y
		lda selptn,x
		ldx zptrNum,y
		sta pt1ptL+0,x
		dey
		bpl -
		rts
ptnpIni	.byte >player.ptn1l,>player.ptn1h,>player.ptn2l,>player.ptn2h,>player.ptn3
ptnpNum	.byte      0,     0,     1,     1,    2
zptrNum .byte      0,     2,     4,     6,    8

getPtIndex	lda #defptlen-1
		sec
		sbc ptnpos
		sbc trkYpos
		rts

dispTable ;display waveform-arpeggio table
		lda #<VRAM+3*40+31
		sta zptr4+0
		lda #>VRAM+3*40+31
		sta zptr4+1
		lda tablepos
		sta zpvar4	;index-count
		lda #13
		sta zpvar5	;row-count
tbdLoop	lda zpvar4
		tax
		and #$0f
		tay
		lda hexdigi,y
		ldy #1
		sta (zptr4),y
		txa
		lsr
		lsr
		lsr
		lsr
		tay
		lda hexdigi,y
+		ldy #0
		sta (zptr4),y
		lda #" "
		cpx tblcnt
		bne +
		lda #">"
+		ldy #2
		sta (zptr4),y
		iny
dWform	lda player.TblCol1,x
		bne dWfNote
dTablFx	lda #"."
		sta (zptr4),y
		iny
		sta (zptr4),y
dTabFx2	iny
		lda player.TblCol3,x
		pha
		lsr
		lsr
		lsr
		lsr
		tax
		lda invdigi,x
		sta (zptr4),y
		iny
		pla
		and #$0f
		tax
		lda invdigi,x
		sta (zptr4),y
		iny
		ldx zpvar4
		lda player.TblCol3,x
		cmp #seqcnt ;test and correct seq-position-FX
		bne +
		lda player.TblCol2,x
		clc
		adc #1
		jmp +++
+		cmp #ptncnt ;test if pattern-position FX, carrect 1F..00 to 00..1F seamlessly
		bne +
		lda #defptlen-1
		sec
		sbc player.TblCol2,x
		jmp ++
+		lda player.TblCol2,x
+		pha
		lsr
		lsr
		lsr
		lsr
		tax
		lda hexdigi,x
		sta (zptr4),y
		iny
		pla
		and #$0f
		tax
		lda hexdigi,x
		sta (zptr4),y
		jmp nexTrow
dWfNote	pha
		lsr
		lsr
		lsr
		lsr
		tax
		lda hexdigi,x
		sta (zptr4),y
		pla
		and #$0f
		tax
		lda hexdigi,x
		iny
		sta (zptr4),y
dTbNote	ldx player.notemode
		bne +
		ldx zpvar4
		jmp dTabFx2
+		ldy zpvar4
		lda player.TblCol2,y
		tax
		lda player.TblCol3,y
		jsr GetNote
		ldy #5
		lda #" "
		sta (zptr4),y
		iny
		lda noteCh1,x
		sta (zptr4),y
		lda noteCh2,x
		iny
		sta (zptr4),y
		lda noteCh3,x
		iny
		sta (zptr4),y
nexTrow	lda #40
		jsr incZpt4
		inc zpvar4
		dec zpvar5
		beq +
		jmp tbdLoop
+		ldy octave
		lda hexdigi+1,y
		sta VRAM+40*1+39
		rts

incZpt1	clc
		adc zptr1+0
		sta zptr1+0
		lda zptr1+1
		adc #0
		sta zptr1+1
		rts
		
incZpt4	clc
		adc zptr4+0
		sta zptr4+0
		lda zptr4+1
		adc #0
		sta zptr4+1
		rts

dispOrderList
		ldx seqpos
		stx zpvar4
		ldy #0
-		ldx zpvar4
		lda #"-"+$80
		cpx marker
		bne +
		lda #"*"+$80		;sign F2-playback marker
+		sta VRAM+24*40,y
		iny
;		lda ptncnt
;		cmp #defptlen-1 ;draw only on pattern-start
;		bne ++
		lda #defaultColour
		cpx seqcnt
		bne +
		lda #13
+		sta CRAM+24*40,y
		sta CRAM+24*40+1,y
+		txa
		pha
		lsr
		lsr
		lsr
		lsr
		tax
		lda invdigi,x
		sta VRAM+24*40,y
		pla
		and #$0f
		tax
		lda invdigi,x
		sta VRAM+24*40+1,y
dSeq1	ldx zpvar4
		lda player.seq1,x
		jsr convSeq
		pha
		;lsr
		;lsr
		;lsr
		;lsr
		;tax
		lda #"0" ;lda hexdigi,x
		sta VRAM+21*40,y
		pla
		and #$0f
		tax
		lda hexdigi,x
		sta VRAM+21*40+1,y
dSeq2	ldx zpvar4
		lda player.seq2,x
		jsr convSeq
		pha
		;lsr
		;lsr
		;lsr
		;lsr
		;tax
		lda #"0" ;lda hexdigi,x
		sta VRAM+22*40,y
		pla
		and #$0f
		tax
		lda hexdigi,x
		sta VRAM+22*40+1,y
dSeq3	ldx zpvar4
		lda player.seq3,x
		jsr convSeq
		pha
		;lsr
		;lsr
		;lsr
		;lsr
		;tax
		lda #"0" ;lda hexdigi,x
		sta VRAM+23*40,y
		pla
		and #$0f
		tax
		lda hexdigi,x
		sta VRAM+23*40+1,y
		inc zpvar4
		iny
		iny
		cpy #39
		bpl +
		jmp -
+		lda #"-"+$80
		sta VRAM+24*40+39
		rts
		
convSeq	lsr
		lsr
		lsr
		lsr
		lsr
		rts

tablepos .byte $00
seqpos .byte $00
ptnpos .byte $00
marker .byte $00

hexdigi	.text "0123456789ABCDEF"
ptNdigi .text "0"+$80,"123","4"+$80,"567","8"+$80,"9AB","C"+$80,"DEF"
invdigi .text $b0,$b1,$b2,$b3,$b4,$b5,$b6,$b7,$b8,$b9,$c1,$c2,$c3,$c4,$c5,$c6


GetNote	;input: A,X - frequency -> determine note
		 CMP #$10
		 BCS HIDET
		 STX zpvar3  ;MULTIPLY SMALLER THAN $10 FREQ
		 ASL zpvar3
		 ROL
		 ASL zpvar3
		 ROL
		 ASL zpvar3
		 ROL
		 ASL zpvar3
		 ROL
		 TAX
p_fback1 LDA FREQBCK,X
		 CMP #96
		 BEQ NDDAT
		 CMP #47
		 BNE LODIS
		 LDA #96+48
LODIS	 SEC
		 SBC #48
NDA		 JMP NDDAT
		 
HIDET	 TAX           ;POINT & READ NOTEDISPLAY-DATA
p_fback2 LDA FREQBCK,X
NDDAT	 TAX
		 RTS

noteCh1 .TEXT "CCDDEFFGGAAHCCDDEFFGGAAHCCDDEFFGGAAHCCDDEFFGGAAHCCDDEFFGGAAHCCDDEFFGGAAHCCDDEFFGGAAHCCDDEFFGGAAH...."
noteCh2 .TEXT "-#-#--#-#-#--#-#--#-#-#--#-#--#-#-#--#-#--#-#-#--#-#--#-#-#--#-#--#-#-#--#-#--#-#-#--#-#--#-#-#-...."
noteCh3	.TEXT "111111111111222222222222333333333333444444444444555555555555666666666666777777777777888888888888...."

FREQBCK
		.BYTE 96,96,96,96,96,96,96,96,96,96,96,96,96,96,96,96 ;THE 0..F WOULD BE IN PLACE, BUT FOR HI, NOT
		.BYTE 47 ; $10 (H-3 246.94Hz) NOTE47
		.BYTE 48 , 49 , 50 , 51 , 52   ;$11..$15 (C-4 ...) HI-NOTE48  / (C-0 ...) LO-NOTE0
		.BYTE 53, 53 , 54 ,55, 55 , 56 ,57, 57 ,58, 58 
		.BYTE  59 ,60, 60 ,61, 61 ,61,62, 62 ,63, 63 ,64, 64 
		.BYTE 64,65, 65 ,65,66, 66 ,66,67, 67 ,67,68, 68 ,68,69, 69 ,69,70,70, 70 
		
		.BYTE 70,71, 71 ,71,72,72, 72 ,72,73,73, 73 ,73,73,74,74, 74 ,74,75,75, 75 ,75,75,76,76, 76 
		.BYTE 76,76,77,77, 77 ,77,77,78,78,78, 78 ,78,78,79,79,79, 79 ,79,79,80,80,80, 80 ,80,80,80,81,81,81, 81 ,81,81,81,82,82,82, 82 
		.BYTE 82,82,82,83,83,83, 83 ,83,83,83,84,84,84,84, 84 ,84,84,84,85,85,85,85, 85 ,85,85,85,85,86,86,86,86, 86 ,86,86,86,86,87,87,87,87, 87 ,87,87,87,87,88,88,88,88,88, 88 
		.BYTE 88,88,88,88,89,89,89,89,89, 89 ,89,89,89,89,89,90,90,90,90,90, 90 ,90,90,90,90,90,91,91,91,91,91,91, 91 ,91,91,91,91,91,91,92,92,92,92,92,92, 92 
		.BYTE 92,92,92,92,92,92,93,93,93,93,93,93, 93 ,93,93,93,93,93,93,94,94,94,94,94,94,94, 94 
		.BYTE 94,94,94,94,94,94,96
		
FreqTablePalLo:
                ;      C   C#  D   D#  E   F   F#  G   G#  A   A#  B
	            .byte 0
                .byte $17,$27,$39,$4b,$5f,$74,$8a,$a1,$ba,$d4,$f0,$0e  ; 1
                .byte $2d,$4e,$71,$96,$be,$e8,$14,$43,$74,$a9,$e1,$1c  ; 2
                .byte $5a,$9c,$e2,$2d,$7c,$cf,$28,$85,$e8,$52,$c1,$37  ; 3
                .byte $b4,$39,$c5,$5a,$f7,$9e,$4f,$0a,$d1,$a3,$82,$6e  ; 4
                .byte $68,$71,$8a,$b3,$ee,$3c,$9e,$15,$a2,$46,$04,$dc  ; 5
                .byte $d0,$e2,$14,$67,$dd,$79,$3c,$29,$44,$8d,$08,$b8  ; 6
                .byte $a1,$c5,$28,$cd,$ba,$f1,$78,$53,$87,$1a,$10,$71  ; 7
                .byte $42,$89,$4f,$9b,$74,$e2,$f0,$a6,$0e,$33,$20,0    ; 8

FreqTablePalHi:
;                      C   C#  D   D#  E   F   F#  G   G#  A   A#  B
		        .byte 0
                .byte $01,$01,$01,$01,$01,$01,$01,$01,$01,$01,$01,$02  ; 1
                .byte $02,$02,$02,$02,$02,$02,$03,$03,$03,$03,$03,$04  ; 2
                .byte $04,$04,$04,$05,$05,$05,$06,$06,$06,$07,$07,$08  ; 3
                .byte $08,$09,$09,$0a,$0a,$0b,$0c,$0d,$0d,$0e,$0f,$10  ; 4
                .byte $11,$12,$13,$14,$15,$17,$18,$1a,$1b,$1d,$1f,$20  ; 5
                .byte $22,$24,$27,$29,$2b,$2e,$31,$34,$37,$3a,$3e,$41  ; 6
                .byte $45,$49,$4e,$52,$57,$5c,$62,$68,$6e,$75,$7c,$83  ; 7
                .byte $8b,$93,$9c,$a5,$af,$b9,$c4,$d0,$dd,$ea,$f8,0    ; 8

;NTSC frequency table for NTSC machines
;------------------------------------------
FREQBCK_NTSC
		.BYTE 96,96,96,96,96,96,96,96,96,96,96,96,96,96,96,96 ;THE 0..F WOULD BE IN PLACE, BUT FOR HI, NOT
		.BYTE 48 ; $10 (C-4) NOTE48
		.BYTE 49 , 50 , 51 , 51 , 52   ;$11..$15 (C#4 ...) HI-NOTE48  / (C#0 ...) LO-NOTE0
		.BYTE 53, 54 , 54 ,55, 56 , 56 ,57, 58 ,58, 59 ;$16..$1F
		.BYTE  59 ,60, 60 ,61, 61 ,62,62, 63 ,63, 63 ,64, 64 ;$20..$2b 
		.BYTE 65,65, 65 ,66,66, 66 ,67,67, 67 ,68,68, 68 ,69,69, 69 ,70,70,70, 70 ;$2c..$3e 
		
		.BYTE 71,71, 71 ,71,72,72, 72 ,72,73,73, 73 ,73,74,74,74, 74 ,75,75,75, 75 ,75,76,76,76, 76 ;$3F..$57
		.BYTE 76,77,77,77, 77 ,77,78,78,78,78, 78 ,78,79,79,79,79, 79 ,79,80,80,80,80, 80 ,80,81,81,81,81,81, 81 ,81,82,82,82,82,82, 82 ;$58..$7c
		.BYTE 82,83,83,83,83,83, 83 ,83,83,84,84,84,84,84, 84 ,84,84,85,85,85,85,85, 85 ,85,85,86,86,86,86,86,86, 86 ,86,86,87,87,87,87,87,87, 87 ,87,87,87,88,88,88,88,88,88, 89 ;$7d..$9f
		.BYTE 89,89,89,89,89,89,89,89,89, 89 ,90,90,90,90,90,90,90,90,90,90, 91 ,91,91,91,91,91,91,91,91,92,92,92, 92 ,92,92,92,92,92,92,92,92,93,93,93,93, 93 ;$a0..$cd
		.BYTE 93,93,93,93,93,93,93,94,94,94,94,94, 94 ,94,94,94,94,94,94,95,95,95,95,95,95,95, 95 ;$ce..$e8
		.BYTE 95,95,95,95,95,95,96, 96,96,96,96,96,96,96,96,96,96,96,96,96,96 ;$e9...$fd

FreqTableNtscLo
                ;      C   C#  D   D#  E   F   F#  G   G#  A   A#  B
                .byte 0
                .byte $0c,$1c,$2d,$3f,$52,$66,$7b,$92,$aa,$c3,$de,$fa  ; 1
                .byte $18,$38,$5a,$7e,$a4,$cc,$f7,$24,$54,$86,$bc,$f5  ; 2
                .byte $31,$71,$b5,$fc,$48,$98,$ee,$48,$a9,$0d,$79,$ea  ; 3
                .byte $62,$e2,$6a,$f8,$90,$30,$dc,$90,$52,$1a,$f2,$d4  ; 4
                .byte $c4,$c4,$d4,$f0,$20,$60,$b8,$20,$a4,$34,$e4,$a8  ; 5
                .byte $88,$88,$a8,$e0,$40,$c0,$70,$40,$48,$68,$c8,$50  ; 6
                .byte $10,$10,$50,$c0,$80,$80,$e0,$80,$90,$d0,$90,$a0  ; 7
                .byte $20,$20,$a0,$80,$00,$00,$c0,$00,$20,$a0,$20,$40  ; 8

FreqTableNtscHi
                ;      C   C#  D   D#  E   F   F#  G   G#  A   A#  B
                .byte 0
                .byte $01,$01,$01,$01,$01,$01,$01,$01,$01,$01,$01,$01  ; 1
                .byte $02,$02,$02,$02,$02,$02,$02,$03,$03,$03,$03,$03  ; 2
                .byte $04,$04,$04,$04,$05,$05,$05,$06,$06,$07,$07,$07  ; 3
                .byte $08,$08,$09,$09,$0a,$0b,$0b,$0c,$0d,$0e,$0e,$0f  ; 4
                .byte $10,$11,$12,$13,$15,$16,$17,$19,$1a,$1c,$1d,$1f  ; 5
                .byte $21,$23,$25,$27,$2a,$2c,$2f,$32,$35,$38,$3b,$3f  ; 6
                .byte $43,$47,$4b,$4f,$54,$59,$5e,$64,$6a,$70,$77,$7e  ; 7
                .byte $86,$8e,$96,$9f,$a9,$b3,$bd,$c9,$d5,$e1,$ef,$fd  ; 8
                

;--------------------------------------------------------------
PALorNTSC .proc ;detect if PAL or NTSC machine, store 0/1 result at a memory location 'PALNTSC'
l1		lda $d012
l2		cmp $d012
		beq l2		;waiting 1 rasterline's time (no matter CPU speed)
		bmi l1		
;the Accumulator here contains the last rasterline that was before $100 ($37:PAL,   $06:NTSC,  $05:oldNTSC)
		sta framemax ;set maximal raster-row number                      %00110111  %00000110   %00000101
		ldx #0		;0 for NTSC
		cmp #(PALfullrast//$100)-1 ;$37	;PAL machine?
		bne +
		inx			;1 for PAL
+		stx PALNTSC ;set to 0 if NTSC, 1 if PAL
		rts
		.pend
		
timecount .proc			;counts the timer if playmode is 1 (songplay mode) - called from IRQ
		lda seconcnt
		bpl nominchg
		ldx PALNTSC
		lda framesec,x
		sta seconcnt
;		lda playmod		;checked in loop already
;		beq retpnt1
		sed				;set decimal mode for timer BCD digits
		lda songtime+1	;seconds
		clc
		adc #1
		sta songtime+1
		cld				;disable decimal mode for timer BCD digits
		cmp #$60
		bne nominchg
		lda #0
		sta songtime+1
		sed				;set decimal mode for timer BCD digits
		lda songtime	;minutes
		clc
		adc #1
		cld				;disable decimal mode for timer BCD digits
		cmp #$60
		bne countfull
		lda #0
countfull sta songtime
nominchg dec seconcnt
		rts
dispTimer
		lda songtime+0
		pha
		lsr
		lsr
		lsr
		lsr
		tay
		lda hexdigi,y
		sta VRAM+40*1+0
		pla
		and #$0f
		tay
		lda hexdigi,y
		sta VRAM+40*1+1
		lda songtime+1
		pha
		lsr
		lsr
		lsr
		lsr
		tay
		lda hexdigi,y
		sta VRAM+40*1+3
		pla
		and #$0f
		tay
		lda hexdigi,y
		sta VRAM+40*1+4
		rts
		.pend

resetimer .proc			;resets the timer
		lda #0
		sta songtime	;reset minutes
		sta songtime+1	;reset seconds
		ldy PALNTSC
		lda framesec,y
		sta seconcnt
		rts
		.pend

songtime .byte 0,0 ;minutes, seconds (BCD format)
PALNTSC .byte 0 ;PAL or NTSC
SIDmachtype .byte 0 ;can be set for SID export but doesn't affect player.tuneheader's PAL/NTSC machine-type mark
framemax .byte 0
seconcnt .byte PALframes1second
framesec .byte NTSCframes1second, PALframes1second		

followp = followplay.followp

followplay .proc		;if follow-mode is on, this subroutine overrides key navigation and controls patterns
followp lda #0			;self-written variable	;followplay signal, if 0-no follow, if 1-follow orderlist by patterns (actptn)
		beq retpnt0
		jsr invPcnt
		sta ptncnt_inv

		ldx #2
-		lda player.patplay+1 ;pattern-play mode?
		cmp patplyOFF+1 ;if pattern-play mode, don't refresh pattern-numbers
		bne +
		ldy playerPtn,x
		lda $00,y
		sta selptn,x	;go to currently played patterns
+		lda ptncnt_inv ;lda ptncnt	;help from player
		sec
		sbc #9			;row at half of the screen, stay in focus
		bcs +
		lda #0
+		sta folsto1+1
		lda #defptlen-1
		sec
		sbc #7			;row at half of the screen, stay in focus
		cmp ptncnt_inv ;cmp ptncnt
		bcs folsto1
		lda #defptlen-17
		jmp +
folsto1 lda #0			;self-written
		cmp #defptlen-1
		bcs SkiPPos		;don't allow invalid positions
+		sta ptnpos
SkiPPos dex
		bpl -

		lda ptncnt_inv ;lda ptncnt
		clc ;sec
		sbc ptnpos
		bcc retpnt0		;don't allow cursor go above row 0 in display area
		cmp #17
		bcs retpnt0		;but in case, don't allow cursor to go through screen
		sta trkYpos
retpnt0 rts
playerPtn .byte player.ptn1pL+1,player.ptn2pL+1,ptn3pt
ptncnt_inv .byte 0
invPcnt	lda #defptlen-1
		sec
		sbc ptncnt
		rts
		.pend

dispAuthorInfo
		ldy #40
-		lda player.authorinfo-1,y
		cmp #" "
		bne + ;find character before last space
		dey
		bne -
+		sty zpvar1
		lda #40
		sec
		sbc zpvar1
		tay
		lsr ;halve the distance -> center text
		tay
		tax
		lda #" "+$80
-		sta headertxt-1,x
		dex
		bne -
-		lda player.authorinfo,x
		;jsr convASCII
		ora #$80
		sta headertxt,y
		inx
		iny
		cpy #40
		bne -
		rts

;==========================================================================
ScreenLayout
headertxt
.text "1 RASTERLINE TRACKER 0.9-HIT F8 FOR MENU"
.text "00:00---------------------------OCTAVE:0"
.text "OFFS00  T1:P00--T2:P00--T3:P00--PROGRAM-"
settingtxt
.text "SPD:";00  00 ...  00 ...  00 ..  00 .. ---"
.text "    ";    01 ...  01 ...  01 ..  01 .. ---"
.text "PW1:";00  02 ...  02 ...  02 ..  02 .. ---"
.text "WF1:";00  03 ...  03 ...  03 ..  03 .. ---"
.text "SR1:";00  04 ...  04 ...  04 ..  04 .. ---"
.text "    ";    05 ...  05 ...  05 ..  05 .. ---"
.text "PW2:";00  06 ...  06 ...  06 ..  06 .. ---"
.text "WF2:";00  07 ...  07 ...  07 ..  07 .. ---"
.text "SR2:";00  08 ...  08 ...  08 ..  08 .. ---"
.text "    ";    09 ...  09 ...  09 ..  09 .. ---"
.text "PW3:";00  10 ...  10 ...  10 ..  0A .. ---"
.text "AD3:";00  11 ...  11 ...  11 ..  0B .. ---"
.text "SR3:";00  12 ...  12 ...  12 ..  0C .. ---"
setting2txt
.text "        13 ...  13 ...  13 ..  ","-"+$80,"-"+$80,"S"+$80,"L"+$80,"I"+$80,"D"+$80,"E"+$80,"-"+$80,"-"+$80
.text "CTF:00  14 ...  14 ...  14 ..  START: 00"
.text "RSW:00  15 ...  15 ...  15 ..  SPEED: 00"
.text "BVL:00  16 ...  16 ...  16 ..  TARGET:00"
;.text "----------------------------------------"
;.text " 00 00 00 00 00 00 00 00 00 00 00 00 00 "
;.text " 00 00 00 00 00 00 00 00 00 00 00 00 00 "
;.text " 00 00 00 00 00 00 00 00 00 00 00 00 00 "
;.text "-01-02-03-04-05-06-07-08-09-10-11-12-13-"

;=============================MENU======================================
Menu	sei
		lda #$35
		sta $01
		lda #0
		sta $d015
		sta namemode
		sta exitsig
		ldx menuColour
		jsr clearScreen
		lda helpmode
		beq goMenu
goHelp	lda #0
		sta cangohl+1
		sta namemode
		jsr ackIRQ
		lda #<helpirq
		ldx #>helpirq
		sta $fffe
		stx $ffff
		inc helprefr
		cli
helploop
		lda helprefr
		beq +
		jsr dispHelp
+		lda exitsig
		beq helploop
		jmp goPage
helprefr .byte 0 ;if help-text needs refresh
		
goMenu	lda #0
		sta playmode ;stop music
		lda #0 ;jsr player.INIT
		sta SID+$18
		sta CHN1+0
		sta CHN1+1
		sta CHN2+0
		sta CHN2+1
		sta CHN3+0
		sta CHN3+1
-		lda $d012
		bne *-3
		lda $dc01
		cmp #$ff
		bne -
		jsr dispMenu		
menuloop 
		lda #$fe
		cmp $d012
		bne *-3
		jsr dispMenuPt
		jsr KeyHandler
		lda exitsig
		beq menuloop
goPage	ldx #0
		stx exitsig
		cmp #1
		bne +
		jmp Menu 
+		cmp #2
		bne +
		jmp softEdi
+		jmp hotStrt

exitsig .byte 0 ;different return-levels (1-Menu, 2-back to Editor, 3-restart Editor)

helpirq	pha				;store sensitive registers and bank status
		txa
		pha
		tya
		pha
		
     ;lda $dc04   ;check timer A, here it jitters between 7...1
     ;eor #7      ;A=7-A so jitter will be 0...6 in A
     ;sta cor2+1  ;self-writing code, the bpl jump-address = A
cor2 nop ;bpl *+2     ;the jump to timer (A) dependent byte
     bit 0 ;cmp #$c9    ;if A=0, cmp#$c9; if A=1, cmp #$c9 again 2 cycles later
     ;cmp #$c9    ;if A=2, cmp#$c9, if A=3, CMP #$EA 2 cycles later
     ;bit $ea24   ;if A=4,bit$ea24; if A=5, bit $ea, if A=6, only NOP
     ;bit $ea24   ;if A=4,bit$ea24; if A=5, bit $ea, if A=6, only NOP
     
     ldx #15
     dex
     bne *-1
     bit 0

		jsr chkPlay

		lda $dc01
		cmp #$ff ;if no key pressed
		bne cangohl
		lda #1
		sta cangohl+1 ;enable keys
cangohl	lda #0 ;self-written - avoids returning back to editor instantly for F8
		beq +
		jsr KeyHandler
		+
		
retirq	lda #1			;more compatible than asl $d019
		sta $d019
		pla
		tay
		pla
		tax
		pla
		rti

menumode .byte 0
helpmode .byte 0 ;if help is being displayed instead of menupoints

MenuKeyHandler
		cpy #$0f*2 ;left shift?
		beq shiftpr
+		cpy #$34*2 ;right shift?
		bne +
shiftpr	jmp nextkey
+		cpy #$03*2 ;F7 key?
		beq +
		cpy #$3f*2 ;STOP key?
		bne chHelp
+		jmp goBackE
chHelp	lda helpmode
		beq chCurDn
		jmp HelpKeyHandler
chCurDn	cpy #$07*2 ;cursor-down key?
		bne chCurRi
		jsr repeatex
		lda shiftsi
		bne menucUp
menucDn	ldx menupoint
		inx
		cpx menusize
		bne +
		ldx #0
+		stx menupoint
		rts
menucUp	dec menupoint
		bpl +
		ldx menusize
		dex
		stx menupoint
+		rts
chCurRi	cpy #$02*2 ;cursor-right key?
		beq increas
		cpy #$28*2 ;plus key?
		bne chMinus
increas	lda shiftsi
		bne decreas
		ldy menupoint
		cpy #(drivePl-MenuPointTxt)/(MenuPointWidth-MenuPointTxt)
		bne +
		jmp setDrive
+		cpy #(relocPl-MenuPointTxt)/(MenuPointWidth-MenuPointTxt)
		bne toggle
		jmp incReloc
toggle	cpy #(VsyncPl-MenuPointTxt)/(MenuPointWidth-MenuPointTxt)
		bne +
		jmp setVsync
+		cpy #(modelPl-MenuPointTxt)/(MenuPointWidth-MenuPointTxt)
		bne retmkey
		jmp setModel
chMinus	cpy #$2b*2 ;Minu-key?
		bne chRetur
decreas ldy menupoint
		cpy #(drivePl-MenuPointTxt)/(MenuPointWidth-MenuPointTxt)
		bne +
		jmp decdevi
+		cpy #(relocPl-MenuPointTxt)/(MenuPointWidth-MenuPointTxt)
		bne toggle
		jmp decReloc
chRetur	cpy #$01*2 ;Return-key?
		bne chNumbr
goMenuP	lda menupoint
		asl
		tax
		lda menuPtAddr,x
		sta menuJmp+1
		lda menuPtAddr+1,x
		sta menuJmp+2
menuJmp	jmp $1111 ;self-written - jumping to menupoint
chNumbr	tya
		ldx menusize
		dex
-		cmp menuPtKey,x
		bne +
		stx menupoint
		jsr dispMenuPt
		jmp goMenuP
+		dex
		bpl -
retmkey	rts

HelpKeyHandler
		cpy #$01*2 ;Return-key?
		bne +
		lda #0
		sta helpmode
		jmp returnM
+		cpy #$33*2 ;home-key?
		bne +
		lda #<HelpText
		ldx #>HelpText
		sta helpposi+0
		stx helpposi+1
		inc helprefr
		rts
+		cpy #$36*2 ;up-arrow?
		beq helpgdn
		cpy #$37*2 ;slash?
		bne +
		lda shiftsi
		bne helpgup
helpgdn	jsr repeatex
		lda helprefr
		bne retmkey
		ldx #HelpTextPageSize ;help page-up/down row amount
		stx zpvar3
-		jsr ScrHlDn
		dec zpvar3
		bne -
		rts
helpgup	jsr repeatex
		ldx #HelpTextPageSize ;help page-up/down row amount
		stx zpvar3
-		jsr ScrHlUp
		dec zpvar3
		bne -
		rts
+		cpy #$07*2 ;cursor-down key?
		bne retmkey
		jsr repeatex
		lda shiftsi
		bne ScrHlUp
		lda helprefr
		bne retmkey
ScrHlDn	lda helpends ;has end of text been reached? (help from dispHelpTxt)
		bne retmkey
		lda helpposi+0
		ldx helpposi+1
		cpx #>(HelpText_end-HelpTextLastPageSize)
		bcc +
		bne retmkey
		cmp #<(HelpText_end-HelpTextLastPageSize)
		bcs retmkey
+		sta HelpPt2+1
		stx HelpPt2+2
HelpPt2	lda $1111 ;self-written
		cmp #$0d ;enter/line-feed?
		beq foundNR
		inc HelpPt2+1
		bne HelpPt2
		inc HelpPt2+2
		bne HelpPt2
foundNR	lda HelpPt2+1 ;found next row
		clc
		adc #2 ;skip line-feed $0a too
		sta helpposi+0
		lda HelpPt2+2
		adc #0 ;add Carry-flag
		sta helpposi+1
		inc helprefr ;cause refreshment of help-text display
		rts
ScrHlUp	lda #>HelpText
		cmp helpposi+1
		bcc +
		bne rethkey
		lda #<HelpText
		cmp helpposi+0
		bcs rethkey
+		lda helpposi+0
		ldx helpposi+1
		sec
		sbc #3 ;previous line-feed should be skipped in tester
		sta HelpPt3+1
		txa
		sbc #0 ;substract Carry flag
		sta HelpPt3+2
HelpPt3	lda $1111 ;self-written
		cmp #$0d
		beq foundPR
		lda HelpPt3+1
		sec
		sbc #1
		sta HelpPt3+1
		lda HelpPt3+2
		sbc #0 ;substract Carry
		sta HelpPt3+2
		bne HelpPt3
foundPR	lda HelpPt3+1 ;found previous row
		clc
		adc #2 ;skip line-feed $0a too
		sta helpposi+0
		lda HelpPt3+2
		adc #0 ;add Carry flag
		sta helpposi+1
		inc helprefr
rethkey	rts

;------------------menupoints------------------------------------------
menuPtKey .byte $38*2,$3b*2,8*2,$0b*2,$10*2,$13*2,$18*2,$1b*2,$20*2, $0a*2,$1c*2,$14*2, $c0, $12*2,$0e*2,$15*2 ;keycodes 1..9 and A..F for menupoints
menuPtAddr .word loadTune,saveTune,saveExe,saveSID,clearTune,clearTable,setDrive,setReloc,setVsync,setModel,readHelp,goBackE, retmkey, demoTune1,demoTune2,demoTune3

loadTune jmp LoadDialog
saveTune jmp SaveDialog
saveExe  jmp ExportExeDialog
saveSID  jmp ExportSIDdialog
		
clearTune
		jsr RyouSur
		bcc +
		jsr initzak
		jsr player.INIT
		jsr SwitchTune
+		jmp returnM

initzak	lda #0
		ldy #0
-		sta player.ptn1l,y
		sta player.ptn1h,y
		sta player.ptn2l,y
		sta player.ptn2h,y
		sta player.ptn3,y
		iny
		bne -
-		sta player.seq1,y
		sta player.seq2,y
		sta player.seq3,y
		iny
		cpy #seqlen
		bne -
		ldy #(player.tuneheader_end-player.tuneheader)-1
-		lda headerIni,y
		sta player.tuneheader,y
		dey
		bpl -
returnM	lda #1
		sta exitsig
		rts
		
headerIni .byte 0,6, $08,$21,$F5, $07,$41,$C5, $08,$00,$F7, $08,$F1,$1F, $08,$01,$1F, 1,1
        .byte 0,0,0,0,0
		.text"AUTHORNAME : TUNE-TITLE...              "
headerIni_end 
		
clearTable
		jsr RyouSur
		bcc +
		jsr inittbl
		jsr player.INIT
		jsr SwitchTune
+		jmp returnM

inittbl ldy #0
-		lda #0
		sta player.TblCol1,y
		sta player.TblCol2,y
		lda #endcmd ;player end-command
		sta player.TblCol3,y
		iny
		cpy #tablemax
		bne -
		jmp returnM
		
setDrive jsr repeatex
		ldy filedialog.devicen+1
		cpy #15	;max. device number
		iny
		blt setdevi
		ldy #8	;min. device-number
setdevi	sty filedialog.devicen+1
dispdev lda filedialog.devicen+1
		jsr decdis2
		sty drivetx		;display device-number digit1 in menu
		sta drivetx+1	;display device-number digit2 in menu
		rts
decdevi	jsr repeatex
		ldy filedialog.devicen+1
		dey
		cpy #8-1
		bne +
		ldy #15
+		jmp setdevi

setReloc
incReloc lda repecnt		;keyrepeat-counter  --- trick to make it faster
		bne +			;check, if repeat counter is 0
		lda #1 ;alternative repeat-speed for relocation-setting
		sta repecnt		;if 0, reset counter to alternative repeatspeed
		rts 
+		jsr repeatex
		lda #>player.tunedata_end;end of area to be exported - hi-byte
		sec 			;injected by tlr at CSDB
		sbc #>PLAYERADDR;injected by tlr at CSDB - this checks if the tune fits into the remaining range below top of memory (RELOC_MAX_HI)
		clc
		adc #1			;add 1 for safety to be in range
		adc relocaddr
		bcs direloc		;if range passes over top of memory, don't allow further change
		cmp #$ff ;max relocation-address - checked tunelength some rows above
		beq direloc
		inc relocaddr
		jmp direloc
		inc relocaddr
direloc jsr dispReloc
		rts
decReloc lda repecnt		;keyrepeat-counter  --- trick to make it faster
		bne +			;check, if repeat counter is 0
		lda #1 ;alternative repeat-speed for relocation-setting
		sta repecnt		;if 0, reset counter to alternative repeatspeed
		rts 
+	 	jsr repeatex 
		lda relocaddr
		cmp #$02		;min. reloc. address
		beq direloc
		dec relocaddr
		jmp direloc

setVsync jsr repeatex 
		lda SIDmachtype
		eor #1
		sta SIDmachtype
		tax
		jmp dispSystem
		
setModel jsr repeatex
		lda SIDtype
		eor #1
		sta SIDtype
		tax
		jmp dispMod
SIDtype	.byte 0
		
readHelp inc helpmode
		jmp returnM

goBackE	;jsr repeatex
		lda #0
		sta menumode
		sta cangoEd+1 ;avoid jumping back&forth
		jsr dispAuthorInfo
		lda player.bandvolu
		sta SID+$18
		lda #0
		sta cangoEd+1
returnE	lda #2
		sta exitsig
		rts
		
demoTune1 
		lda #<demotune1data
		ldx #>demotune1data
		jmp cpyDemo

demoTune2
		lda #<demotune2data
		ldx #>demotune2data
		jmp cpyDemo
		
demoTune3
		lda #<demotune3data
		ldx #>demotune3data

cpyDemo	sta zptr1+0
		stx zptr1+1
		jsr RyouSur
		bcs +
		jmp returnM
+		lda #<player.tuneheader
		ldx #>player.tuneheader
		sta zptr2+0
		stx zptr2+1
		ldy #0			;forward-motion to prevent overwriting source with target if they overlap (most probably)
DcpyLop	lda (zptr1),y
		sta (zptr2),y
		inc zptr1+0
		bne +
		inc zptr1+1
+		inc zptr2+0
		bne +
		inc zptr2+1
+		lda zptr2+0
		cmp #<player.tunedata_end ;check if copied the entire musicdata - low-byte part
		bne DcpyLop
		lda zptr2+1 ;check if copied the entire musicdata - high-byte part
		cmp #>player.tunedata_end ;until end of musicdata is reached
		bne DcpyLop
		jsr SwitchTune
restrtE	lda #3
		sta exitsig
		rts

decdis2 .proc			;decimal display - input:accu, output: in Accu and Y-register
		ldy #$2f
		sec
-		iny
		sbc #10
		bcs -
		adc #$3a
		rts
		.pend

;--------------------------------------------------------------------
dispMenu
		ldy #39
-		lda MenuScreenLayout,y
		sta VRAM,y
		lda MenuScreenLayout+40,y
		sta VRAM+40,y
		lda #"-"
		sta VRAM+20*40,y
		sta VRAM+23*40,y
		lda MenuHelpTxt,y
		jsr convASCII
		sta VRAM+21*40,y
		lda MenuHelpTxt+40,y
		jsr convASCII
		sta VRAM+22*40,y
		lda CreditTxt,y
		jsr convASCII
		sta VRAM+24*40,y
		dey
		bpl -
		jsr dispdev
		ldx SIDmachtype
		jsr dispSystem
		ldx SIDtype
		jsr dispMod 
		jsr dispReloc
		jsr dispMenuPt
		rts

dispMenuPt
		lda #<MenuPointTxt
		ldx #>MenuPointTxt
		sta zptr1
		stx zptr1+1
		lda #<(VRAM+3*40+13)
		ldx #>(VRAM+3*40+13)
		sta zptr2
		stx zptr2+1
		ldx #0 ;counts rows
dMenuPt	ldy #(MenuPointWidth-MenuPointTxt)-1
-		lda (zptr1),y
		jsr convASCII
		cpx menupoint
		bne +
		ora #$80		;highlight current menupoint
+		sta (zptr2),y
		dey
		bpl -
		lda #(MenuPointWidth-MenuPointTxt)
		jsr incZpt1
		lda #40
		clc
		adc zptr2+0
		sta zptr2+0
		lda zptr2+1
		adc #0
		sta zptr2+1
		inx
		cpx menusize
		bne dMenuPt
		rts

menupoint .byte 0 ;current menupoint
menusize .byte (MenuPointTxt_end-MenuPointTxt)/(MenuPointWidth-MenuPointTxt) 


convASCII	
		cmp #$60		;input: Accu, output: Accu
		bcc +			;handle lowercase
		and #$1f
+		rts
		

dispHelp ldy #39
-		lda HelpFooter,y
		jsr convASCII
		ora #$80
		sta VRAM+23*40,y
		lda HelpFooter+40,y
		jsr convASCII
		ora #$80
		sta VRAM+24*40,y
		dey
		bpl -
		jmp dispHelpTxt

dispHelpTxt
		lda helpposi+0
		ldx helpposi+1
		sta HelpPtr+1
		stx HelpPtr+2
		lda #<VRAM
		ldx #>VRAM
		sta zptr3+0
		stx zptr3+1
		ldx #23 ;counts rows
HelpRow	ldy #0
HlpLoop	lda HelpPtr+2
		cmp #>HelpText_end
		bcc HelpPtr
		lda HelpPtr+1
		cmp #<HelpText_end
		bcc HelpPtr
		inc helpends
		lda #0
		sta helprefr
		rts
HelpPtr	lda $1111 ;source-byte
		cmp #$0d ;Enter?
		bne ShowChr
		lda #" "  ;fill the rest of the line with whitespace
-		cpy #40
		bpl +
		sta (zptr3),y
		iny
		bne -
+		lda #40 ;line-feed
		clc
		adc zptr3+0
		sta zptr3+0
		lda zptr3+1
		adc #0 ;add Carry
		sta zptr3+1
		inc HelpPtr+1
		bne +
		inc HelpPtr+2
+		jmp linefed
ShowChr	cpy #40  ;only the beginning is visible, if longer than 40...
		bcs AdvText
		cmp #$1f	;don't show controller codes
		bcc AdvText
		cmp #$60		;input: Accu, output: Accu
		bcc +			;handle lowercase
		and #$1f
+		sta (zptr3),y
		iny
AdvText	inc HelpPtr+1
		bne +
		inc HelpPtr+2
+		cpy #80 ;safety, allow max. 80 chars/row to prevent endless loops
		bcc HlpLoop
linefed	dex
		bne HelpRow
		lda #0
		sta helpends
		sta helprefr
retHelp	rts 

helpposi .word HelpText ;help text's position
helpends .byte 0 ;notifies keyhandler that help reached its end


;-----------------------------------------------------------------------------
MenuScreenLayout
		.text "     MENU FOR 1 RASTERLINE TRACKER      "
		.text "     -----------------------------      "
MenuPointTxt
		.text "1. Load Tune  "
MenuPointWidth
		.text "2. Save Tune  "
		.text "3. Save Exec. "
		.text "4. Save SID   "
		.text "5. Clear Tune "
		.text "6. Clear Table"
drivePl	.text "7. Drive: "
drivetx .text "08  "
relocPl	.text "8. Reloc:$"
reloctx	.text "1000"
VsyncPl	.text "9. Vsync: "
vsynctx .text "PAL "
modelPl	.text "A. Model: "
modeltx .text "8580"
		.text "B. Read Help  "
		.text "C. Back F8/Esc"
		.text "--------------"
		.text "D. Demo Tune 1"
		.text "E. Demo Tune 2"
		.text "F. Demo Tune 3"
MenuPointTxt_end

MenuHelpTxt
.text "Use key 1..F or Cursor up/down to select"
.text "+/-,Cursor left/right, RETURN to adjust."
CreditTxt
.text "      2013 Hermit Software Hungary      "

.byte $0d,$0a ;ensure enter before 1st row (for up-scroller)
HelpText
.binary "1raster-help.txt"
HelpText_end
HelpTextLastPageSize=810 ;maybe...
HelpTextPageSize=20 ;how much to page-up/down

HelpFooter
.text "Press Cursor up/dn or / to scroll text.."
.text "Press RETURN to Menu, F8/STOP to editor."

dispMod ldy #3			;display and set SID-model based on X register
-		lda NewSIDtxt,y
		cpx #1			;check if 8580
		beq +			;branch if 8580
		lda OldSIDtxt,y
+		sta modeltx,y
		lda SIDflags+1
		and #%11001111
		ora ModelFlag,x ;SID-model
		sta SIDflags+1
		dey
		bpl -
		rts
ModelFlag .byte $10,$20 ;SID-model bits to SIDflags
OldSIDtxt .text "6581"
NewSIDtxt .text "8580"
PALNTSCor .byte %1000,%0100 ;PAL/NTSC values in SID-header
MachTypeTx .text "NTSC","PAL "

dispSystem ;input is machine-type in X
		lda SIDflags+1 ;sign PAL/NTSC in SID-header
		and #%11110011
		ora PALNTSCor,x
		sta SIDflags+1
		txa
		asl
		asl
		tay
		ldx #0
-		lda MachTypeTx,y
		sta vsynctx,x
		iny
		inx
		cpx #4
		bne -
		rts

detectSIDtype			;X is 0 at return if 6581, 1 if 8580
		sei				;No disturbing interrupts
		lda #$ff
		cmp $d012		;Don't run it on a badline.
		bne *-3
		lda #$ff		;Set frequency in voice 3 to $ffff
		sta SID+$12 ;...and set testbit (other bits doesn't matter) in $d012 to disable oscillator
		sta SID+$0e
		sta SID+$0f
		lda #$20		;Sawtooth wave and gatebit OFF to start oscillator again.
		sta SID+$12
		lda SID+$1b ;Accu now has different value depending on sid model (6581=3/8580=2)
		lsr				;...that is: Carry flag is set for 6581, and clear for 8580.
		ldx #1			;transfer Carry-flag to X
		bcc +
		dex				;X=0
+		jsr dispMod		;display and set SID-Model in SID header, based on X register
		rts

dispReloc
		lda relocaddr
		pha
		lsr
		lsr
		lsr
		lsr
		tay
		lda hexdigi,y
		sta reloctx+0
		pla
		and #$0f
		tay
		lda hexdigi,y
		sta reloctx+1
		rts

RyouSur					;Ask 'are you sure', wait for Y/N and set Carry accordingly (1 if Y)
RyouXpos = 11
RyouYpos = 9
RyouCol = 1
		ldy #16
-		lda #"*"		;DECOR
		sta VRAM+40*(RyouYpos+0)+RyouXpos,y
		sta VRAM+40*(RyouYpos+3)+RyouXpos,y
		lda #RyouCol
		sta CRAM+40*(RyouYpos+0)+RyouXpos,y
		sta CRAM+40*(RyouYpos+1)+RyouXpos,y
		sta CRAM+40*(RyouYpos+2)+RyouXpos,y
		sta CRAM+40*(RyouYpos+3)+RyouXpos,y
		lda RyouSurTx,y
		sta VRAM+40*(RyouYpos+1)+RyouXpos,y
		lda RyouSurT2,y
		sta VRAM+40*(RyouYpos+2)+RyouXpos,y
		dey
		bpl -
		jmp YesOrNo
RyouSurTx .text "* ARE YOU SURE? *"
RyouSurT2 .text "*  (Y/N) (I/N)  *"

YesOrNo					;wait for Y/N and set Carry accordingly (1 if Y)
		sei
		lda #$e7
		sta $dc00
YNloop	lda $dc01
		cmp #($ff-2)		;Y key (or I key)?
		bne +
-		cmp $dc01		;wait to release key
		beq -
		sec
		rts
+		cmp #($ff-$80)	;N (or V) key?
		bne YNloop		;key-testing loop
-		cmp $dc01		;wait to release key
		beq -
		clc
		rts

;------------------------- FILENAME HANDLER ----------------------------------------
;ROM routine jumptable addresses
SETNAM = $ffbd ;Set file name parameters. Input: A = File name length; X/Y = Pointer to file name. (Real address: $FDF9)
SETLFS = $ffba ;Set file parameters. Input: A = Logical number; X = Device number; Y = Secondary address. (Real address: $FE00)
OPEN   = $ffc0 ;Open file. (Must call SETLFS and SETNAM beforehands.) Used registers: A, X, Y. (Real address: ($031A), $F34A)
CLOSE  = $ffc3 ;Close file. Input: A = Logical number. Used registers: A, X, Y. (Real address: ($031C), $F291.)
CHKIN  = $ffc6 ;Define file as default input. (Must call OPEN beforehands.) Input: X = Logical number. Used registers: A, X. (Real address: ($031E), $F20E)
CHRIN  = $ffcf ;Read byte from default input (for keyboard, read a line from the screen). (If not keyboard, must call OPEN and CHKIN beforehands.) Output: A = Byte read. Used registers: A, Y. (Real address: ($0324), $F157.)
CHKOUT = $ffc9 ;Open a channel for output (Registers affected:  A, X), input: A=data, X = logical file number , Real address: ($0320), $F250.
CHROUT = $ffd2 ;(BSOUT) Write byte to default output. (If not screen, must call OPEN and CHKOUT beforehands.) Input: A = Byte to write. (Real address: ($0326), $F1CA)
CLRCHN = $ffcc ;Close default input/output files (for serial bus, send UNTALK and/or UNLISTEN); restore default input/output to keyboard/screen. Used registers: A, X. (Real address: ($0322), $F333)
SCINIT = $ff81 ;Initialize VIC; restore default input/output to keyboard/screen; clear screen; set PAL/NTSC switch and interrupt timer. Used registers: A, X, Y. (Real address: $FF5B.)
SAVE   = $ffd8 ;SAVE. Save file. (Must call SETLFS and SETNAM beforehands.) Input: A = Address of zero page register holding start address of memory area to save; 
               ;X/Y = End address of memory area plus 1. Output: Carry: 0 = No errors, 1 = Error; A = KERNAL error code (if Carry = 1). (Real address: $F5DD.)
LOAD   = $ffd5 ;LOAD. Load or verify file. (Must call SETLFS and SETNAM beforehands.) Input: A: 0 = Load, 1-255 = Verify; X/Y = Load address (if secondary address = 0).
               ;Output: Carry: 0 = No errors, 1 = Error; A = KERNAL error code (if Carry = 1); X/Y = Address of last byte loaded/verified (if Carry = 0). Used registers: A, X, Y. (Real address: $F49E.)  
READST = $ffb7 ;Fetch status of current device, value of ST variable. (For RS232, status is cleared.),No Input , Output: A = Device status., Used registers: A.,Real address: $FE07.
;Indirect ROM addresses (possibly used by cartridge fastloaders)
IOPEN=$031A ;Indirect entry to Kernal OPEN Routine ($F34A).
ICLOSE=$031C ;Indirect entry to Kernal CLOSE Routine ($F291).
ICHKIN=$031E ;Indirect entry to Kernal CHKIN Routine ($F20E).
ICKOUT=$0320 ;Indirect entry to Kernal CHKOUT Routine ($F250).
ICLRCH=$0322 ;Indirect entry to Kernal CLRCHN Routine ($F333).
IBASIN=$0324 ;Indirect entry to Kernal CHRIN Routine ($F157).
IBSOUT=$0326 ;Indirect entry to Kernal CHROUT Routine ($F1CA).
ICLALL=$032C ;Indirect entry to Kernal CLALL Routine ($F32F).
ILOAD=$0330 ;Indirect entry to Kernal LOAD Routine ($F4A5).
ISAVE=$0332 ;Indirect entry to Kernal SAVE Routine ($F5ED).

;--------------
LoadDialog
		sei
PRGlist	ldx menuColour
		jsr ClearScreen
		ldx #2			;set SID-Wizard module extension to import
-		lda ORMext,x
		sta filedialog.swftype+1,x
		dex
		bpl -
		lda #2
		sta filedialog.defawin+1	;file-selector is preferred default subwindow in filer (but changeable with TAB)
		lda #0
		sta filedialog.editauth	;init author-editing possibility (disable by default)
		jsr filedialog.reredir		;read directory, typing filename, return filename in 'namebuf'
		bcc +
		jmp retMenu		;if Carry=1 on exit, restart file-handler (no returning to anywhere, like in tracker)
;load tune (notify if unsuccessful) - fastloader may fit in memory (and could be selected with Shift+Return)
+		jsr filedialog.iniwfn		;(secondary address inside = 0 ensures load-address forcing)
		jsr OPEN		;open file for checking version in its beginning
		jsr READST		;read drive status
		cmp #0			;if status is 0, no error
		beq OKload		;check if file exists
fnotfnd jsr filedialog.filenot		;display file not found and wait for SPACE
		jsr filedialog.clsfile		;in case of error, close to allow further trials
backfil	jmp PRGlist
OKload	;setting FILE-descriptor (SETLFS in 'iniwfn') and OPEN must have happened before
		ldx #$01		;	file #1
		jsr CHKIN		; - call chkin to set input file #.
		jsr READST
		bne fnotfnd
		jsr CHRIN		;get load-address lo-byte
		jsr READST
		bne fnotfnd
		jsr CHRIN		;get load-address hi-byte
		sta relocaddr
		lda #<PLAYERADDR ;Set Start Address for saver loop
		ldx #>PLAYERADDR
		sta zptr1+0
		stx zptr1+1
skiplop	jsr CHRIN
		inc zptr1+0
		bne +
		inc zptr1+1
+		lda zptr1+0	;check if end of player
		cmp #<player.tunedata
		lda zptr1+1
		sbc #>player.tunedata
		bcc skiplop
lZakDat	lda #<player.tunedata
		ldx #>player.tunedata
LoadDat	sta loadptr+1 ;start-address
		stx loadptr+2
loadloop				;save effective file-content byte-by-byte
		jsr CHRIN
loadptr	sta $1111		;self-written, used as 'pointer' for read byte to store
		inc loadptr+1
		bne +
		inc loadptr+2
+		lda loadptr+1	;check if end of file
		cmp #<player.tunedata_end ;expoendadd+0
		lda loadptr+2
		sbc #>player.tunedata_end ;expoendadd+1
		bcc loadloop
		;jsr READST
		;bne ExitErr ;error?
		ldx #$00		; filenumber 0
		jsr CHKIN
		jsr filedialog.clsfile		;close file #1
		;check PAL/NTSC, and if differs, offer conversion of notes/pitches
		jsr SwitchTune ;initialize data/cursor positions
		jmp restrtE ;to editor

SaveDialog  ;save the little player with the music data to be ready for inclusion in programs
		sei
		jsr Validate  ;unmute possibly muted channels and disable pattern-play
ORMlist	ldx menuColour
		jsr ClearScreen
		ldx #2			;set SID-Wizard module extension to import
-		lda ORMext,x
		sta filedialog.swftype+1,x
		dex
		bpl -
		lda #0
		sta filedialog.defawin+1	;file-selector is preferred default subwindow in filer (but changeable with TAB)
		lda #1
		sta filedialog.editauth	;switch on author-editing for saver
		jsr filedialog.reredir		;read directory, typing filename, return filename in 'namebuf'
		bcs retMenu			;if Carry=1 on exit, restart file-handler (no returning to anywhere, like in tracker)
		jsr appendPRGW	;append PRG file Writing string to filename
		jsr OpenExp
		bcs ORMlist ;error?
		lda #<PLAYERADDR
		jsr CHROUT		;address lo-byte
		lda relocaddr
		jsr CHROUT		;address hi-byte
		jsr relocate
		lda #<PLAYERADDR ;Set Start Address for saver loop
		ldx #>PLAYERADDR
		jsr SaveDat
		php ;store Carry
		jsr unrelocate
		plp ;restore Carry
		bcs ORMList ;error?
retMenu	jmp returnM

ExportExeDialog
		sei
		jsr Validate  ;unmute possibly muted channels and disable pattern-play
ExeList	ldx menuColour
		jsr ClearScreen
		ldx #2			;set SID-Wizard module extension to import
-		lda EXEext,x
		sta filedialog.swftype+1,x
		dex
		bpl -
		lda #0
		sta filedialog.defawin+1	;file-selector is preferred default subwindow in filer (but changeable with TAB)
		lda #1
		sta filedialog.editauth	;switch on author-editing for saver
		jsr filedialog.reredir		;read directory, typing filename, return filename in 'namebuf'
		bcs retMenu
		jsr appendPRGW	;append PRG file Writing string to filename
		jsr OpenExp
		bcs ExeList ;error?
		lda #$01		;set C64 BASIC-program start address $0801 to run standalone
		jsr CHROUT
		lda #$08
		jsr CHROUT
		lda #<EXEstarter;Set Start Address
		ldx #>EXEstarter
		jsr SaveDat
		bcs ExeList ;error?
		jmp returnM

ExportSIDdialog
		sei
		jsr Validate  ;unmute possibly muted channels and disable pattern-play
SIDlist	ldx menuColour
		jsr ClearScreen
		ldx #2			;set SID-Wizard module extension to import
-		lda SIDext,x
		sta filedialog.swftype+1,x
		dex
		bpl -
		lda #0
		sta filedialog.defawin+1	;file-selector is preferred default subwindow in filer (but changeable with TAB)
		lda #1
		sta filedialog.editauth	;switch on author-editing for saver
		jsr filedialog.reredir		;read directory, typing filename, return filename in 'namebuf'
		bcs retMenu
		;jsr SetSIDply	;set SID player-pointers, etc
		jsr SetSIDt		;set SID-text
		jsr appendSEQW	;append SEQ file Writing string to filename
		jsr OpenExp
		bcs SIDlist ;error?
		jsr reloSID
		jsr SaveSIDheader
		php
		jsr unreloSID
		plp
		bcs SIDlist
		jsr relocate
		lda #<PLAYERADDR ;Set Start Address  (skip CIA-starter code, it's not needed for 1x speed)
		ldx #>PLAYERADDR
		jsr SaveDat
		php ;store Carry
		jsr unrelocate
		plp ;restore Carry
		bcs SIDList ;error?
		jmp returnM

;--------------------------
OpenExp	;init workfile name pointer and length, bank, and device,file,channel
		lda #$36		;init workfile name pointer and length, bank, and device,file,channel
		sta $01
		lda bufnln2		;appended filename-length
		ldx #<filedialog.namebuf	;point to appended (,S,W or ,P,W) filename
		ldy #>filedialog.namebuf
		jsr SETNAM		;init filename pointer and length, and device,file,channel
		lda #1			;	file # 1
		ldx filedialog.devicen+1	;self-written, device # 8 is default (modifiable by user)
		ldy #2			;	channel # 2 ;this can be used for SEQ file formats
		jsr SETLFS		; - call setlfs
		jsr OPEN		;open the file for writing
		bcc OKtoSav		;check if file exists, if exists, go forward with saving
ExitErr	jsr filedialog.saverror	;display save-error and wait for keypress
		jsr filedialog.clsfile		;in case of error, close to allow further trials
		sec  ;sign error to caller
		rts
OKtoSav	ldx #$01		;	file #1
		jsr CHKOUT		; - call chkout to set output file #.
		clc ;sign 'no error' to caller
		rts

;----------------------------------------------
SaveDat	sta zptr1+0 ;startaddress (by A and X)
		stx zptr1+1
		ldy #0			;reset Y to always be 0 while saving (no need to index, zp-pointers increase instead)
saveloop				;save effective file-content byte-by-byte
		jsr READST
		bne ExitErr ;error?
+		lda (zptr1),y
		jsr CHROUT		;write byte to disk
		inc zptr1+0
		bne +
		inc zptr1+1
+		lda zptr1+0	;check if end of file
		cmp #<player.tunedata_end ;expoendadd+0
		lda zptr1+1
		sbc #>player.tunedata_end ;expoendadd+1
		bcc saveloop
		ldx #$00		; filenumber 0
		jsr CHKOUT		; call CHKOUT (reset output device to prevent keyboard-typings being written to disk)
		jsr filedialog.clsfile		;close file #1
expdone clc ;sign OK to caller
		rts

SaveSIDheader
		ldy #0			;write SID-header to disk ($7C bytes)
-		jsr READST
		bne ExitErr
		lda SIDHEADER,y
		jsr CHROUT		;write byte to disk
		iny
		cpy #SIDHEADERSIZE
		bne -
		lda #<PLAYERADDR ;write load-address to SID
		jsr CHROUT		;address lo-byte
		lda relocaddr
		jsr CHROUT		;address hi-byte
hdrdone	clc
		rts

;------------------------------------------
appendSEQW .proc		;append SEQ file Writing string to filename
		lda #"S"
		.byte $2c		;jump over lda #
		.cerror appendPRGW & 0
		.pend

appendPRGW .proc		;append PRG file Writing string to filename
		lda #"P"
		sta PRGwriTxt+1

		ldx filedialog.bufnlen+1	;length of selected/typed filename with extension
		ldy #0
-		lda PRGwriTxt,y
		beq +			;end of string?
		sta filedialog.namebuf,x
		sta $8000,x
		inx
		iny
		bne -
+		stx bufnln2		;adjust filename-length to be expanded with ",P,W"
		rts
PRGwriTxt .null ",P,W"	;the appended modifiers after the filename for PRG-type outputs
		.pend

;set SID text (author name and tune-title)
SetSIDt .proc
		ldx #0			;will show the position of ':' - if no separator, value will stay 0
		ldy #39
-		lda player.authorinfo,y
		cmp #":"		;if there's a ':' separator, the string before will be handled as author-name
		bne +
		tya				;only the 1st ':' will be used as split-separator
		tax
+		dey
		bpl -
		cpx #0
		bne splitTX		;if ':' was found
noSplit ldy #31			;SID-header text field max. size
-		lda #0			;reset author-info text
		sta SIDauthrtx,y
		lda player.authorinfo,y
		sta SIDtitletx,y
		dey
		bpl -
		rts
splitTX cpx #32-1
		bpl noSplit		;if the separator is at illegal place (authorname would be longer than allowed 32
		lda #0
		sta SIDauthrtx,x;put '0' string-termination
		txa
		tay
-		lda player.authorinfo-1,y;right before ':'
		sta SIDauthrtx-1,y
		dey
		bne -
		ldy #0
-		lda player.authorinfo+1,x;right after ':'
		sta SIDtitletx,y
		iny
		cpy #32-1		;max text field-length in SID-header
		beq +
		inx
		cpx #40
		bne -
+		lda #0			;put '0' string-termination
		sta SIDtitletx,y
		rts
		.pend

SwitchTune ;jsr player.INIT
		lda #0
		sta instPos
		sta marker
		sta selptn+0
		sta selptn+1
		sta selptn+2
		sta seqpos
		sta ptnpos
		sta tablepos
		sta trkYpos
		sta ptrack
		sta tblYpos
		sta otrack
		sta oliXpos
		sta diginum
		sta followp+1
		lda #1
		sta window
		jsr dispAuthorInfo
Validate ;unmute possibly muted channels and disable pattern-play
		ldx #0
		lda #1
		jsr MuteSub ;unmute channel 1
		ldx #1
		lda #1
		jsr MuteSub ;unmute channel 2
		ldx #2
		lda #1
		jsr MuteSub ;unmute channel 3
		jsr PtPlayOFF ;disable pattern-play mode
		rts
		
;---------------------------
relocate lda relocaddr	;calculate difference
		sec
		sbc #>PLAYERADDR
		sta relocdiff
		beq endrelo ;if no need for relocation
relodata ldx #0
reltadd	lda reloctable,x ;self-written, address of relocation-table
		sta zptr1+0	;zeropage lo-pointer set
		inx
reltad2	lda reloctable,x;self-written, address of relocation-table
		sta zptr1+1	;zeropage hi-pointer set
		beq endrelo	;end of table found?
		ldy #2	;we modify only 2nd (hi) address-byte of absolute-addressing 6502 instructions in player code
		lda (zptr1),y
		clc
		adc relocdiff
		sta (zptr1),y
		inx	;go to next relocation-pointer
		bne reltadd
		inc reltadd+2	;in case relocation table would be bigger than 256 bytes
		inc reltad2+2
		bne reltadd	;jump
endrelo	rts ;end of relocation-table reached
unrelocate	;set everything back to relative of PLAYERADDR (resume relocation, not to affect further exports)
		lda #0	;negate relocation-difference
		sec
		sbc relocdiff	;calculate difference backwards
		beq endrelo
		sta relocdiff	;reset reloc.address
		jmp relodata

;set SID-header jump-addresses too
reloSID	lda relocaddr	;calculate difference
		sec
		sbc #>PLAYERADDR
		sta relocdiff
		beq endrelo ;if no need for relocation
SIDjAdd	lda SIDinitadd+0
		clc
		adc relocdiff
		sta SIDinitadd+0
		lda SIDplayadd+0
		clc
		adc relocdiff
		sta SIDplayadd+0
		rts
unreloSID lda #0	;negate relocation-difference
		sec
		sbc relocdiff	;calculate difference backwards
		beq endrelo
		sta relocdiff	;reset reloc.address
		jmp SIDjAdd
		
;------------------------- variables used by relocator and file-handler -------------------
relocaddr .byte >PLAYERADDR ;relocation target address HI-byte
relocdiff .byte $00	;hi-address relative to base address

bufnln2 .byte 0			;the length of the appended filename
ORMext	.text "ORM"		;importable SID-Wizard module extension
exportext				;extensions for export-types
EXEext	.text "EXE"		;2
SIDext	.text "SID"		;3

;---------------------------------------------------------------------------
filedialog	.proc			;input:Accu - extension read directory, typing filename, return filename in 'namebuf'

ddevice=8 ;default disk device
dirxpos= 1 ;filelist X screen position
dirypos= 4 ;filelist Y screen position
maxnamelen=12 ;maximum filename-length to display
extlen = 4 ;extension-length (with dot)
authorilen=authorinfo_size ;max. length of author-info that can be displayed & typed
namecolums= 3 ;number of filename-columns (screenwidth divided by max. filename-length
dirvmem= VRAM+dirypos*40+dirxpos ;Video-RAM base memory address of filelist
disknam= VRAM+40*24+5 ;Video-RAM place of diskname $ disk-ID (disk-header)
diskfree=VRAM+40*24+35 ;place of free disk-place display
filenamp=VRAM+10 ;filename-position on screen
authorip=VRAM+40*2 ;author-info screen-position
namesep= $22 ; " filename-delimiter character code
filelistheight=19 ;height of file-lister/selector subwindow
filelstmax= filelistheight*namecolums ;maximum length of filelist that can be displayed on 1 screen
RETURNcode=13 ;return/enter (carreturn) charactercode  (signs end of directory)

reredir sei				;init ROM $ VICbank and read disk directory to the screen
		lda #$36		;enable KERNAL ROM (only)
		sta $01
		lda #0
		sta $d01a
		lda #3
		sta $dd00
		ldy #39			;draw UI base
-		lda filetx1,y
		sta VRAM+0,y;filename and type (extension)
		lda #$62+$80	;$78  ;$71 ;$68
		sta VRAM+40,y;separator line 1
		sta VRAM+40*3,y;separator line 2
		sta VRAM+40*23,y;separator line 3
		lda player.authorinfo,y
skipaut sta authorip,y	;author info and title
+		lda filetx2,y
		sta VRAM+40*24,y;diskname and free space
		dey
		bpl -
		lda #$17		;small character set
		sta $d018

readdir lda #$01		;	filename-length is 1
		ldx #<dirname	;	lo byte pointer to file name. ("$")
		ldy #>dirname	;	hi byte pointer to file name. ("$")
		jsr inifnam		;init filename memory position and length
		jsr OPEN		; - call open
		bcs NOredir		;if error
		jsr READST
		bne NOredir		;if error
		ldx #$01		;	file #1 ;if no error (device is present), just go forward
		jsr CHKIN		; - call chkin to set input file #.
;read and display disk-header
		jsr CHRIN
		cmp #$C3		;normally first byte is $01, VICE gives $C3 if device is not present
		bne +			;if Carry remained set, no device is present
NOredir	jsr nodevic		;if error opening device, notify user
		jmp typesel		;and skip filelist-making, still give a chance to return to e.g. editor and change drive-number
+		ldx #6
-		jsr CHRIN		;read through some unneeded characters (amount given in X)
		dex
		bne -
		ldx #0			;read diskname & ID
		stx dirready	;signs if directory fully read
-		jsr CHRIN
		beq listfil
		ora #$80
		cpx #24			;don't allow too long diskheader to be displayed (crop beginning to max. length)
		bpl +
		sta disknam,x	;standard 'jsr CHROUT' is not good for special display modes
+		inx
		bne -
;parse and display directory for filesizes and names
listfil jsr clrfsel		;clear file-selector area, init file-listing variables
flistlo ldx #3			;file-lister loop
-		jsr CHRIN		;get filesize (last one will be filesize in blocks)
		sta sizebuf,x	;sizebuf might be displayed or filesizes stored in a filesize-list to help user calculate
		dex
		bpl -
-		jsr CHRIN		;check for beginning of filename (1st " delimiter)
		cmp #RETURNcode ;return/enter signal - check for end of dir/filelist
		beq closdir		;if end of filelist, close device
		cmp #namesep
		bne -			;search for left " filename-delimiter
		ldy #0			;start checking&copying filename
-		jsr CHRIN
		cmp #RETURNcode ;return/enter signal - check for end of dir/filelist
		beq closdir		;if end of filelist, close device
		cmp #namesep	;check for filename end (right " filename-delimiter)
		beq frowend		;find $00 row-delimiter
		sta namebuf,y	;loads actual filename to buffer
		iny
		cpy #17			;maximum allowed filename-length for C64 devices
		bne -
frowend jsr CHRIN		;check for $00 - end of line of filename, then comes beginning of next line pointer and filesize/freeblocks info
		bne frowend
chekext lda filecnt
		cmp #filelstmax ;check displayable filelist max. length
		bcs typesel		;if reached all displayable content, go to type/select name or read next dir. page
		;or bcs flistlo ;if reached, no further displaying, just checking rows for filesizes, etc.
		dey				;check name for .SWM/.SWI extension
		ldx #extlen-1	;check 4 character long (with dot) filename-extension to filter on filetypes (.SWM/.SWI)
-		lda namebuf,y
		cmp swftype,x
		bne flistlo		;if there was any difference between extension and end of filename-string, don't display filename
		dey
		bmi flistlo		;if filename is empty (only extension), no point to display
		dex
		bpl -
dispnam lda namebuf,y
		cmp #$20		;is last character SPACE? -then sign it with a delimiter character
		bne dispnlp		;if not SPACE, simply display filename
		lda #$20+$40	;the invisible space will sign real end of 'space-ended' filename to loader/saver routine
		bne +			;jump
dispnlp lda namebuf,y	;filename-displayer loop
+		sta (zptr1),y
		dey
		bpl dispnlp
		inc filecnt		;count displayed filenames
nextrow jsr cycfcol		;cycle filer-column counter, Y output is tabbing amount ;go to next screenplace for next filename
		jmp flistlo		;back to read next line

closdir inc dirready	;sign that directory fully read
		lda sizebuf+1	;display free kilobytes on disk in decimal format (divide blocks by 4)
		lsr sizebuf+0	;divide by 2
		ror
		lsr sizebuf+0	;divide by 2 (4 in sequence)
		ror
		ldx #100		;convert
		jsr divideX
		sty diskfree+0
		ldx #10
		jsr divideX
		sty diskfree+1
		tax
		lda hexdigi,x
		sta diskfree+2
		jsr clsfile

;-------type filename or author info / select file to load/save--------------
typesel sei				;type/select filename & description
		lda #$35
		sta $01
		lda #$fe
		sta $dc00
-		lda $dc01		;wait for keys to be released, especially RETURN
		cmp #$ff
		bne -
		lda #2			;set filename-typer mode in key-handler
		sta namemode
defawin lda #0			;self-written - default filer-window (selector is preferred in case of loader)
		beq +
		jsr setsele		;set filename-selector subwindow as default for loader
		jmp fkeylop
+		jsr setfnam		;set filename-typer subwindow by default
fkeylop lda #$ff
		cmp $d012		;constant 50Hz timing loop for keyboard-handler and cursor-displayer
		bne *-3
		lda #0
		sta fkeybout	;reset keyhandler output
		jsr KeyHandler		;handles key-repeating and name-mode
		jsr disfcur		;display filer-cursors
		lda fkeybout	;keyhandler-output
		cmp #$f8		;F1 key check
		bne +			;if F1 pressed, re-read directory
		jsr regauth		;clsfile ;close dir., save typed author-info to re-read filelist
		jmp reredir		;readdir ;re-read directory/file-list
+		cmp #$f9		;F3/UP(PageDown) key check
		bne +			;if F3/Pagedown pressed, flip to next page of disk-directory (if exists)
		lda dirready	;check, if directory fully read
		bne lopover		;if directory read entirely, no further paging
		jmp listfil		;further read filelist
+		cmp #$fe		;if stop/esc pressed, returning to editor (abort file-operation)
		bne +			;$fe/$ff
		jsr regauth		;clsfile ;close directory-file - register author-text
		sec				;cause file-dialog abort in main routine (restart editor)
		rts
+		cmp #$ff		;if RETURN/Enter pressed, take action (load/save)
		bne lopover
		ldy filerwindow
		cpy #1
		beq +
		jsr regname		;register filename in 'namebuf' and author-info
		clc				;cause load/save in main routine, if return pressed window 1 or 2 (filename/file-selector)
		rts
+		jsr setfnam		;if window 1, go back to filename-typer
lopover jmp fkeylop

;----------------------------------file-dialog subroutines----------------------------------
iniwfn	lda #$36		;init workfile name pointer and length, bank, and device,file,channel
		sta $01
bufnlen lda #0			;self-written code - length of selected/typed filename
		ldx #<namebuf
		ldy #>namebuf
inifnam jsr SETNAM		;init filename pointer and length, and device,file,channel
devicen ldx #ddevice	;self-written, device # 8 is default (modifiable by user)
		txa
		jsr decdis2
		sty $07c3
		sta $07c4
		lda #1			;	file # 1
		ldy #0			;	channel # 0 - (in case of LOAD: if 1, load to address automatically, if 0, address can be given in X/Y)
		jmp SETLFS		; - call setlfs

;---------------------------------------------------------------
clrfsel					;clear file-selector area, init file-listing variables
		lda #$36
		sta $01
		lda #<dirvmem
		ldx #>dirvmem
		jsr inicptr		;set video-pointers
		ldx #0			;clear filelist-area
		stx filecnt		;counts displayed filenames
		stx zpvar3	;counts columns
fsclrlp ldy #40-2
		lda #$20		;Space character
-		sta (zptr2),y
		dey
		bpl -
		lda #40			;one row forward
		jsr incptrs		;increase zptr2	 ;iny -> Y=0 -> preserve zptr1
		cpx #filelistheight
		bne fsclrlp
		rts
;-----------------------------------------------------------------------------
filenamer				;handles filename-typing/selection (called from keyer/nametyper routine in keyhandler.inc, char passed in Accu, keycode in Y)
		sta fkeybout	;for main routine
		cmp #$fa		;Control/TAB keys - cycle filer sub-windows
		bne filkeys
		ldy filerwindow
		bne +			;if filename-subwindow
		iny				;set Y to 1
		lda editauth	;if loader, no author-info editing, skipping that
		beq setsele		;if player.authorinfo editing disabled (=0) go straight to selector
		lda #authorilen-1
		sta zpvar4	;length of input-field
		lda #<authorip
		ldx #>authorip
		jmp setyfld		;set typer field
+		cpy #1
		beq setsele		;if 1 set filename-selector subwindow
		clc
		jsr diselcu		;first clear selector-cursor
		jmp setfnam		;then cycle back to filename-typer window
setsele lda filecnt		;set file-selector
		beq setfnam		;if filelist is empty, don't go to selector, go back to / stay at filename-typer instead
		lda #<dirvmem
		ldx #>dirvmem
		jsr inicptr		;set video-pointers
		sec				;set display-mode for cursor
		jsr diselcu		;display file-selector cursor
		ldy #2			;set to 2
		bne setyfld		;jump
setfnam lda #maxnamelen-1
		sta zpvar4	;length of input-field
		lda #<filenamp
		ldx #>filenamp
		ldy #0
setyfld sty filerwindow ;branched to here if filer-window was 2
setyfl2 jsr inidptr		;takes A , X
		lda #0
		sta zpvar5	;reset cursor-position
		sta zpvar3	;counts filename-selector columns
		rts

filkeys ldx filerwindow ;handle keys depending on subwindow---------------------------------
		cpx #2
		bne typefld		;if not file-selector window, go to typer ;if file-selector subwindow, use cursor keys and Return only
		cmp #$fb		;cursor up/down
		bne +
		jsr fiselLR		;3 columns
		jsr fiselLR
		jmp fiselLR
+		cmp #$fc		;cursor left/right
		bne endfiln
fiselLR clc				;clear cursor
		jsr diselcu		;clear file-selector-cursor
		jsr fiselec		;change cursor-pointer if possible
		sec				;draw cursor
		jmp diselcu		;draw file-selector-cursor
fiselec lda shiftsi		;the effective algorithm itself to select file
		bne fselbwd
fselfwd ldy zpvar5
		iny
		cpy filecnt
		bcs endfiln
		inc zpvar5
cycfcol inc zpvar3	;column-tabbing counter
		ldx zpvar3
		cpx #namecolums
		bne +
		ldx #0
		stx zpvar3
+		ldy colutab,x
		jmp incompt
fselbwd lda zpvar5	;filename-count
		beq endfiln
		dec zpvar5
		ldx zpvar3
		ldy colutab,x
		dec zpvar3	;cycle filer-column counter
		bpl +			;if column-turnaround, compensate non-integer divisibility of screen width per filename-columns
		ldx #namecolums-1
		stx zpvar3
+		jmp decptrs

typefld ldy zpvar5	;handle typing fields------------------------------------------------
		cmp #$f5
		bne +			;Home-key?
		lda #0
		sta zpvar5
		rts
+		cmp #$f7		;insert?
		beq +
		cmp #$fd		;backspace?
		bne cursidw
		lda shiftsi		;Insert? (Shift+Backspace)
		beq bkspclp
+		ldy zpvar4
-		cpy zpvar5
		beq clrcurp
		dey
		lda (zptr2),y;shif letters forward
		iny
		sta (zptr2),y
		dey
		bpl -
clrcurp lda #$20
		sta (zptr2),y
endfiln rts
bkspclp jsr fcurbwd
		ldy zpvar5
-		cpy zpvar4
		beq clrcurp		;BACKSPACE character to last cursorplace
		iny
		lda (zptr2),y;shift letters back
		dey
		sta (zptr2),y
		iny
		jmp -
cursidw cmp #$fc		;cursor-sideways keys?
		bne norchar
		lda shiftsi
		beq fcurfwd
fcurbwd cpy #0			;move cursor backwards in name-field
		beq endfiln
		dec zpvar5
		rts
norchar cmp #0
		bmi endfiln		;avoid typing characters for remaining control characters
		sta (zptr2),y;normal character branch
fcurfwd cpy zpvar4	;move cursor forward (if possible)
		bpl endfiln
		inc zpvar5
		rts

;--------------------------------------------------------------------------------
disfcur					;display cursor over typed data
		jsr clrcurs		;clear type-fields from cursor invertions
		lda filerwindow
		cmp #2			;selector?
		beq +
		ldy zpvar5	;display text-typer cursor
		lda (zptr2),y
		ora #$80
		sta (zptr2),y
+		rts
diselcu ldy #maxnamelen-1;display/clear file-selector cursor (action depends on Carry-bit, displays if Carry=1, clears if Carry=0)
-		lda (zptr1),y
		and #$7f		;clear cursor
		bcc +			;if Carry bit set
		sta filenamp,y
		ora #$80		;set cursor
+		sta (zptr1),y
		dey
		bpl -
		rts
clrcurs ldy #maxnamelen-1;clear type-fields from cursor invertions
-		lda filenamp,y
		and #$7f
		sta filenamp,y
		dey
		bpl -
		ldy #authorilen-1;clear cursor
-		lda authorip,y
		and #$7f
		sta authorip,y
		dey
		bpl -
		rts

;--------------------------
regname jsr clrcurs		;start by clearing type-fields from cursor invertions
		ldy #maxnamelen-1;copy/register typed/selected filename and author-info (latter only if editauth is on
-		lda filenamp,y	;loop search back to first non-Space character
		cmp #$20		;check for spaces (if filename is 'spaced;, $60 Space will be noticed)
		bne +			;if not space coming, we found end of filename (without extension), and we can add the extension to it
		dey
		bpl -			;if not looped here, empty filename
+		iny
		sty zpvar3	;copy filename before last space
		beq +			;zero filename
		ldy #0
-		lda filenamp,y
		sta namebuf,y
		iny
		cpy zpvar3	;stored length of filename
		bne -
+		ldx #0			;add extension
-		lda swftype,x	;read extension-string
		sta namebuf,y	;add it to namebuf.
		iny
		inx
		cpx #extlen
		bne -
		sty bufnlen+1	;register buffered name's length
regauth lda editauth
		beq clsfile		;if author-info being edited, register it in tune-header
		ldy #authorilen-1;hardwired author-info length
-		lda authorip,y
		sta player.authorinfo,y
		dey
		bpl -
clsfile lda #$36
		sta $01
		lda #1			;	file # to close
		jsr CLOSE		; - so close it
		jmp CLRCHN		; - clear all channels

nodevic					;if OPEN failed, warn user and ask for retrial
		ldy #28			;display 'device not present'	(and later ask for overwrite-confirmation)
-		lda nodevicep,y
		sta VRAM+40*24,y
		dey
		bpl -
		rts ;jmp waitspace
		
filetx1 .text "FILENAME:             "
		.byte $75		;the ruler between filename and extension display
		.text "(FILETYPE: *";.
swftype .text ".ORM)"	;SW-file type - string to search extension in filenames to filter on tune/instrument
filetx2 .text "DR:   .....................   FREE:...KB"
dirname .byte $24		;.asc "$" ;$ is the name of disk-directory

;scratch .text "S0:" ;Scratch-signal (OPEN 15,8,15,"S0:file name":CLOSE 15) - will preceed filename when user want to overwrite a file (Save&Replace "@filename" is buggy in old 1541 drives)
namebuf .fill 16		;filename-buffer (for extension-checking and for disk-operations)
		.fill 10		;for safety reasons, if filename gets appended by commands

sizebuf .fill 4			;(byte 0 and 1) filesize-temp-storage, after reading dir., free blocks (last 2 bytes (row-pointers) not needed)
colutab .byte maxnamelen+2,maxnamelen+1,maxnamelen+1;column-tabbing values (to get screenwidth as sum) (14+13+13=40)
filecnt .byte 0			;amount of displayed files in directory
dirready .byte 0		;value is 1 if directory is fully read
editauth .byte 0		;0 - no author info display $ edition, 1 - autor info displayed & editable
filerwindow .byte 0		;window in file-menu: 0-filename-typer , 1-player.authorinfo-typer, 2-file-selector
fkeybout .byte 0		;keyboard-routine output of filer's keyboard handler - used by main loop to take some actions

;error messages in filer window, 29 character long each
nodevicep .text " NO DISK OR DRIVE PRESENT!!! "
fnotfound .text " FILE NOT FOUND! PRESS SPACE."
;filexists .text "ERROR! (EXISTS?) PRESS SPACE."
saveerror .text " SAVING ERROR 0! PRESS SPACE. "

filenot ;jsr clsfile		;in case of error, close to allow further trials
		ldy #28			;Display 'file not found' message and wait for SPACE
-		lda fnotfound,y
		sta VRAM+40*24,y
		dey
		bpl -
		jmp waitspace

saverror ldy #28			;Display 'save error' message and wait for SPACE
-		lda saveerror,y
		sta VRAM+40*24,y
		dey
		bpl -
		jsr READST		;read status error-code
		clc
		adc #"0"		;shift numbering for status-number display to be PETSCII compatible
		sta VRAM+40*24+14;display status error-code
		jmp waitspace	;wait for SPACE key press/release

.pend

waitspace .proc				;wait for SPACE press and release
		sei
		lda #$7f
		sta $dc00
-		lda $dc01		;wait for pressing SPACE
		cmp #$ff
		beq -
-		lda $dc01		;wait to release SPACE
		cmp #$ff
		bne -
		rts
		.pend

divideX stx zpvar2	;divide A by divisor X, remainder is in A for further division, Y gets charcode to display
		ldx #0
-		sec
		cmp zpvar2	;compare divisor
		bcc +
		sbc zpvar2	;substract divisor
		inx
		bne -			;jump
+		
		ldy hexdigi,x
		rts

;!these pointer routines are used by many other parts of the code , don't touch them, not even rearrange them!!!
inicptr					;lo-pointer in A, hi-pointer in X, set zeropage pointers - order of entry points is important!
		sta zptr3
		stx zptr3+1
ctempzp lda zptr3
		sta zptr1
		ldx zptr3+1
		stx zptr1+1
inidptr sta zptr2
		stx zptr2+1
		rts

ctotmpz lda zptr1
		sta zptr3
		lda zptr1+1
		sta zptr3+1
		rts

incptrs					;increase source zeropage pointers by boundary size in A, and target pointer by Y value
		iny
incopt2 clc
		adc zptr2
		sta zptr2
		bcc incompt
incptnp inc zptr2+1
incompt tya
incmpt2 clc
		adc zptr1
		sta zptr1	;set compressed data pointer to next position
		bcc +
		inc zptr1+1
+		inx
		rts

decptrs tya
deccptr sta zpvar1
		lda zptr1
		sec				;input: Accu -decrease 16bit player.tunedata-pointers by value given in a
		sbc zpvar1	;self-written
		sta zptr1
		bcs +
dechptr dec zptr1+1
+		dey
		rts

nametyper				;subroutine which handles nametyping (instrument-naming or file-naming)
		lsr
		tay
		jsr repeatex
		lda keychar,y	;(from datatables.inc)
		beq inmret1		;no process if 0
		cpx #2			;check if file-dialog is opened
		bne +
		jmp filedialog.filenamer	;jump filename-typer (in menu.inc)
+
inmret1 rts

namemode .byte 0		;if value is 2, filename-typing/selection mode

;keyboard-charactertable for naming								  ;- 01 - 02 - 04 - 08 - 10 - 20 - 40 - 80 ---
keychar	.byte $fd,$ff	,$fc   ,$fe   ,$f8	 ,$f9	,$fe   ,$fb;;; del |retu|crri| F7 | F1 | F3 | F5 |crdn|fe
		.byte 51 ,23+64 ,1+64  ,52	  ,26+64 ,19+64 ,5+64  ,$f6;;;	3  | W	| A  | 4  | Z  | S	| E  |lSH |fd
		.byte 53 ,18+64 ,4+64  ,54	  ,3+64  ,6+64	,20+64 ,24+64;	5  | R	| D  | 6  | C  | F	| T  | X  |fb
		.byte 55 ,25+64 ,7+64  ,56	  ,2+64  ,8+64	,21+64 ,22+64;	7  | Y	| G  | 8  | B  | H	| U  | V  |f7
		.byte 57 ,9+64	,10+64 ,48	  ,13+64 ,11+64 ,15+64 ,14+64;	9  | I	| J  | 0  | M  | K	| O  | N  |ef
		.byte 43 ,16+64 ,12+64 ,45	  ,46	 ,$3a	,$00   ,$2c;;;	+  | P	| L  | -  | .  | :	| @  | ,  |df
		.byte $f7,0		,$3b   ,$f5   ,$f6	 ,$3d	,$f9   ,$2f;;;pound| *	| ;  |HOME|rSH | =	| up | /  |bf
		.byte 49 ,$fe	,$fa   ,50	  ,32	 ,0		,17+64 ,$fe;;;	1  |back|CTRL| 2  |SPC | C= | Q  |stop|7f

;------------------------------------------------------
;copy-buffers
noteBufL .fill defptlen,0
noteBufH .fill defptlen,0
noteBufLen .byte 0 ;length of copied data
pt3buf	.fill defptlen,0 
pt3bufLen .byte 0 ;length of copied data

;example tunes
demotune1data
.binary "lightforce.orm",2+(player.tunedata-player.playercode)
demotune1data_end
demotune2data
.binary "demotune2.orm",2+(player.tunedata-player.playercode)
demotune2data_end
demotune3data
.binary "visage.orm",2+(player.tunedata-player.playercode)
demotune3data_end

.align $100
dummySID .fill $20,0 ;write muted sounds here...

;===========================================================================
;TODO: 
;-create transpose? detune? in patterns?
;-convert PAL/NTSC frequency values if created on other machine type
;-optimize player speedwise into 63 cycles (sizewise should stay in $100)
;-measure current and max. cycles?
